<?php
/**
 * Handles ajax requests.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

use Everest_Backup\Modules\Backup_Config;
use Everest_Backup\Modules\Backup_Content;
use Everest_Backup\Modules\Backup_Database;
use Everest_Backup\Modules\Backup_Plugins;
use Everest_Backup\Modules\Backup_Themes;
use Everest_Backup\Modules\Backup_Uploads;
use Everest_Backup\Modules\Migration;
use Everest_Backup\Modules\Restore_Config;
use Everest_Backup\Modules\Restore_Content;
use Everest_Backup\Modules\Restore_Database;
use Everest_Backup\Modules\Restore_Multisite;
use Everest_Backup\Modules\Restore_Plugins;
use Everest_Backup\Modules\Restore_Themes;
use Everest_Backup\Modules\Restore_Uploads;
use Everest_Backup\Modules\Restore_Users;

/**
 * Handles ajax requests.
 *
 * @since 1.0.0
 */
class Ajax {


	/**
	 * Init ajax.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function __construct() {
		add_action( 'wp_ajax_everest_backup_backup_init', array( $this, 'backup_init' ) );
		add_action( 'wp_ajax_everest_backup_backup_database', array( $this, 'backup_database' ) );
		add_action( 'wp_ajax_everest_backup_backup_uploads', array( $this, 'backup_uploads' ) );
		add_action( 'wp_ajax_everest_backup_backup_themes', array( $this, 'backup_themes' ) );
		add_action( 'wp_ajax_everest_backup_backup_plugins', array( $this, 'backup_plugins' ) );
		add_action( 'wp_ajax_everest_backup_backup_content', array( $this, 'backup_content' ) );
		add_action( 'wp_ajax_everest_backup_backup_final', array( $this, 'backup_final' ) );

		add_action( 'wp_ajax_everest_backup_export', array( $this, 'export_files' ) );
		add_action( 'wp_ajax_everest_backup_import', array( $this, 'import_files' ) );
		add_action( 'wp_ajax_everest_backup_abort', array( $this, 'abort_backup' ) );
		add_action( 'wp_ajax_everest_backup_addon', array( $this, 'install_addon' ) );
	}

	/**
	 * Set/Get temp session data.
	 *
	 * @param array|object|null $data Data to save temporarily.
	 * @param bool              $get If passed true, then it will return the data otherwise it will save it.
	 * @return mixed
	 */
	protected function temp( $data = null, $get = false ) {
		if ( ! session_id() ) {
			session_start();
		}

		if ( $get ) {
			return isset( $_SESSION['ebwp_ajax_temp'] ) ? everest_backup_sanitize_array( $_SESSION['ebwp_ajax_temp'] ) : null;
		}

		$_SESSION['ebwp_ajax_temp'] = $data;

		if ( ! $data ) {
			if ( isset( $_SESSION['ebwp_ajax_temp'] ) ) {
				unset( $_SESSION['ebwp_ajax_temp'] );
			}

			return;
		}

	}

	/**
	 * Handle the process abort requests.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function abort_backup() {
		Logs::init( 'abort' );

		$response = everest_backup_get_ajax_response( 'everest_backup_abort' );

		if ( ! isset( $response['type'] ) ) {
			$message = __( 'Process type not found.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		if ( 'backup' !== $response['type'] ) {
			$message = __( 'Abort is only compatible with backup.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		$backup_file = everest_backup_get_current_archive();

		if ( ! file_exists( $backup_file ) ) {
			everest_backup_send_success();
		}

		unlink( $backup_file );

		Logs::done( __( 'Backup aborted.', 'everest-backup' ) );
		everest_backup_send_success();
	}

	/**
	 * Init backup.
	 */
	public function backup_init() {
		Logs::init( 'backup' );

		wp_cache_flush();

		everest_backup_setup_environment();

		$timer_start = time();

		/**
		 * Verify nonce and get response.
		 */
		$params = everest_backup_get_ajax_response( 'everest_backup_backup_init' );

		Logs::info( __( 'Initializing backup', 'everest-backup' ) );

		Logs::info( sprintf( __( 'Backup started at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_start ) ) );

		$compress = new Compress( $params );

		$this->temp(
			array(
				'compress'    => $compress,
				'timer_start' => $timer_start,
				'params'      => $params,
			)
		);

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_database',
				'message'     => __( 'Backup started. Exporting database.', 'everest-backup' ),
			)
		);
	}

	/**
	 * Backup database.
	 *
	 * @return void
	 */
	public function backup_database() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_database' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress = $temp['compress'];
		$params   = $temp['params'];

		Backup_Database::init( $compress );
		Backup_Config::init( $compress );

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_uploads',
				'message'     => ! empty( $params['ignore_database'] ) ? __( 'Database ignored. Exporting media files.', 'everest-backup' ) : __( 'Exporting media files.', 'everest-backup' ),
			)
		);

	}

	/**
	 * Backup media files.
	 *
	 * @return void
	 */
	public function backup_uploads() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_uploads' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress = $temp['compress'];
		$params   = $temp['params'];

		Backup_Uploads::init( $compress );

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_themes',
				'message'     => ! empty( $params['ignore_media'] ) ? __( 'Media files ignored. Exporting theme files.', 'everest-backup' ) : __( 'Exporting theme files.', 'everest-backup' ),
			)
		);

	}

	/**
	 * Backup theme files.
	 *
	 * @return void
	 */
	public function backup_themes() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_themes' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress = $temp['compress'];
		$params   = $temp['params'];

		Backup_Themes::init( $compress );

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_plugins',
				'message'     => ! empty( $params['ignore_themes'] ) ? __( 'Theme files ignored. Exporting plugin files.', 'everest-backup' ) : __( 'Exporting plugin files.', 'everest-backup' ),
			)
		);

	}

	/**
	 * Backup plugin files.
	 *
	 * @return void
	 */
	public function backup_plugins() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_plugins' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress = $temp['compress'];
		$params   = $temp['params'];

		Backup_Plugins::init( $compress );

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_content',
				'message'     => ! empty( $params['ignore_themes'] ) ? __( 'Plugin files ignored. Exporting remaining files from wp-content folder.', 'everest-backup' ) : __( 'Exporting remaining files from wp-content folder.', 'everest-backup' ),
			)
		);

	}

	/**
	 * Backup wp-content files.
	 *
	 * @return void
	 */
	public function backup_content() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_content' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress = $temp['compress'];

		Backup_Content::init( $compress );

		$all_files  = $compress->get_files();
		$total_size = $compress->get_total_filesize();

		wp_send_json(
			array(
				'status'      => 'in-process',
				'next_action' => 'everest_backup_backup_final',
				/* translators: Here, %1$d is the number of total files and %2$s is the uncompressed size of total files. */
				'message'     => sprintf( __( 'Your website backup is almost ready. Archiving %1$d files ( %2$s ).', 'everest-backup' ), count( $all_files ), size_format( $total_size ) ),
			)
		);

	}

	/**
	 * Archive files and send response.
	 *
	 * @return void
	 */
	public function backup_final() {

		wp_cache_flush();

		everest_backup_setup_environment();

		/**
		 * Verify nonce and get response.
		 */
		everest_backup_get_ajax_response( 'everest_backup_backup_final' );

		$temp = $this->temp( null, true );

		/**
		 * \Everest_Backup\Compress class object.
		 *
		 * @var \Everest_Backup\Compress
		 */
		$compress    = $temp['compress'];
		$timer_start = $temp['timer_start'];

		$zip = $compress->zip();

		everest_backup_set_current_archive( $zip );

		Filesystem::init()->clean_storage_dir();

		if ( ! $zip ) {
			$message = __( 'Unable to create backup file.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		$timer_end = time();

		Logs::info( sprintf( __( 'Backup completed at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_end ) ) );

		$total_time = human_time_diff( $timer_start, $timer_end );
		Logs::info( sprintf( __( 'Total time:', 'everest-backup' ) . ' %s', $total_time ) );

		Logs::info( sprintf( __( 'Backup size:', 'everest-backup' ) . ' %s', size_format( filesize( $zip ) ) ) );

		$zipurl = everest_backup_convert_file_path_to_url( $zip );

		$everest_backup_migration = new Migration(
			array(
				'file'       => basename( $zip ),
				'auto_nonce' => true,
			)
		);

		Logs::done( __( 'Backup completed', 'everest-backup' ) );
		everest_backup_send_success(
			array(
				'zipurl'        => $zipurl,
				'migration_url' => $everest_backup_migration->get_url(),
			)
		);
	}

	/**
	 * Initialize our backup related work.
	 *
	 * @since 1.0.0
	 */
	public function export_files() {
		Logs::init( 'backup' );

		wp_cache_flush();

		everest_backup_setup_environment();

		$timer_start = time();

		$params = everest_backup_get_ajax_response( 'everest_backup_export' );

		Logs::info( __( 'Initializing backup', 'everest-backup' ) );

		Logs::info( sprintf( __( 'Backup started at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_start ) ) );

		$compress = new Compress( $params );

		/**
		 * * Here, order is very important for the backup process.
		 *
		 * =====================
		 * * Default Order:
		 * =====================
		 *
		 * * Backup_Database
		 * * Backup_Config
		 * * Backup_Uploads
		 * * Backup_Themes
		 * * Backup_Plugins
		 * * Backup_Content
		 */
		Backup_Database::init( $compress );
		Backup_Config::init( $compress );
		Backup_Uploads::init( $compress );
		Backup_Themes::init( $compress );
		Backup_Plugins::init( $compress );
		Backup_Content::init( $compress );

		$zip = $compress->zip();

		everest_backup_set_current_archive( $zip );

		Filesystem::init()->clean_storage_dir();

		if ( ! $zip ) {
			$message = __( 'Unable to create backup file.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		$timer_end = time();

		Logs::info( sprintf( __( 'Backup completed at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_end ) ) );

		$total_time = human_time_diff( $timer_start, $timer_end );
		Logs::info( sprintf( __( 'Total time:', 'everest-backup' ) . ' %s', $total_time ) );

		Logs::info( sprintf( __( 'Backup size:', 'everest-backup' ) . ' %s', size_format( filesize( $zip ) ) ) );

		$zipurl = everest_backup_convert_file_path_to_url( $zip );

		$everest_backup_migration = new Migration(
			array(
				'file'       => basename( $zip ),
				'auto_nonce' => true,
			)
		);

		Logs::done( __( 'Backup completed', 'everest-backup' ) );
		everest_backup_send_success(
			array(
				'zipurl'        => $zipurl,
				'migration_url' => $everest_backup_migration->get_url(),
			)
		);
	}

	/**
	 * Initialize our restore related work.
	 *
	 * @since 1.0.0
	 */
	public function import_files() {
		if ( everest_backup_doing_rollback() ) {
			Logs::init( 'rollback' );
		} else {
			Logs::init( 'restore' );
		}

		if ( ! current_user_can( 'upload_files' ) ) {
			$message = __( 'Current user does not have permission to upload files.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		wp_cache_flush();

		everest_backup_setup_environment();

		everest_backup_get_ajax_response( 'everest_backup_import' );

		$timer_start = time();

		Logs::info( sprintf( __( 'Restore started at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_start ) ) );

		$extract = new Extract(); // @phpcs:ignore

		/**
		 * * Here, order is very important for the restoration process.
		 *
		 * =====================
		 * * Default Order:
		 * =====================
		 *
		 * * Restore_Config
		 * * Restore_Multisite
		 * * Restore_Database
		 * * Restore_Users
		 * * Restore_Uploads
		 * * Restore_Themes
		 * * Restore_Plugins
		 * * Restore_Content
		 */
		Restore_Config::init( $extract );
		Restore_Multisite::init( $extract ); // Only works if current site is a multisite.
		Restore_Database::init( $extract );
		Restore_Users::init( $extract ); // Only works if current site is a multisite.
		Restore_Uploads::init( $extract );
		Restore_Themes::init( $extract );
		Restore_Plugins::init( $extract );
		Restore_Content::init( $extract );

		Filesystem::init()->clean_storage_dir();

		Logs::info( __( 'Flushing unnecessary cache.', 'everest-backup' ) );

		$extract->flush_cache();

		$timer_end = time();

		Logs::info( sprintf( __( 'Restore completed at:', 'everest-backup' ) . ' %s', wp_date( 'h:i:s A', $timer_end ) ) );

		$total_time = human_time_diff( $timer_start, $timer_end );
		Logs::info( sprintf( __( 'Total time:', 'everest-backup' ) . ' %s', $total_time ) );

		Logs::done( __( 'Restore completed.', 'everest-backup' ) );

		wp_clear_auth_cookie();
		everest_backup_send_success();
	}

	/**
	 * Install and activate free addon from the addon page.
	 *
	 * @return void
	 */
	public function install_addon() {

		$plugins_dir = WP_PLUGIN_DIR;
		$response    = everest_backup_get_ajax_response( 'everest_backup_addon' );

		wp_cache_flush();

		$addon_category = ! empty( $response['addon_category'] ) ? $response['addon_category'] : '';
		$addon_slug     = ! empty( $response['addon_slug'] ) ? $response['addon_slug'] : '';

		$addon_info = everest_backup_addon_info( $addon_category, $addon_slug );

		$package = $addon_info['package'];

		$plugin_folder = $plugins_dir . DIRECTORY_SEPARATOR . $addon_slug;
		$plugin_zip    = $plugin_folder . '.zip';
		$plugin        = $addon_slug . '/' . $addon_slug . '.php';

		$data = wp_remote_get(
			$package,
			array(
				'sslverify' => false,
			)
		);

		$content = wp_remote_retrieve_body( $data );

		if ( ! $content ) {
			wp_send_json_error();
		}

		if ( file_exists( $plugin_zip ) ) {
			unlink( $plugin_zip );
		}

		Filesystem::init()->writefile( $plugin_zip, $content );

		if ( ! file_exists( $plugin_zip ) ) {
			wp_send_json_error();
		}

		if ( is_dir( $plugin_folder ) ) {
			/**
			 * Plugin directory already exists, then delete the existing plugin directory first.
			 */
			Filesystem::init()->delete( $plugin_folder, true );
		}

		unzip_file( $plugin_zip, $plugins_dir );
		activate_plugin( $plugin );
		unlink( $plugin_zip );

		wp_send_json_success();
	}
}

new Ajax();
