<?php
/**
 * Core abstract class for handling zip files.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Core abstract class for handling zip files.
 *
 * @since 1.0.0
 */
abstract class Archiver {

	/**
	 * Temporary data added during import or export process, so that other modules can use it during the process.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $temps = array();

	/**
	 * Type of library we are currenty using.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $type;

	/**
	 * Compression library object.
	 *
	 * @var \ZipArchive|\PclZip
	 * @since 1.0.0
	 */
	protected $lib;

	/**
	 * Zipfile custom extension.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $extension = EVEREST_BACKUP_BACKUP_FILE_EXTENSION;

	/**
	 * Full path to the zip file with filename and extension concatenated.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $zipname;

	/**
	 * Files to add for compression.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $files;

	/**
	 * Estimated total file size of all the files in the list.
	 *
	 * @var int
	 * @since 1.0.0
	 */
	private $total_files_size = 0;

	/**
	 * Files paths to replace inside zip.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $paths_to_replace;

	/**
	 * Init class.
	 *
	 * @param string $zipname Full path to the zip file with filename but without file extension.
	 * @since 1.0.0
	 */
	public function __construct( $zipname ) {

		if ( ! $zipname ) {
			return;
		}

		$this->zipname = wp_normalize_path( $zipname . $this->extension );

		$paths_to_replace = wp_parse_args(
			$this->paths_to_replace,
			array(
				WP_CONTENT_DIR,
			)
		);

		$paths_to_replace       = array_map( 'trailingslashit', $paths_to_replace );
		$this->paths_to_replace = array_map( 'wp_normalize_path', $paths_to_replace );

		if ( class_exists( 'ZipArchive' ) ) {

			/**
			 * Default PHP compression library.
			 */
			$this->type = 'ziparchive';
			$this->lib  = new \ZipArchive();
		} else {

			if ( ! class_exists( 'PclZip' ) ) {
				require ABSPATH . 'wp-admin/includes/class-pclzip.php';
			}

			/**
			 * If ZipArchive is not available.
			 *
			 * @see wp-admin/includes/class-pclzip.php
			 */
			$this->type = 'pclzip';
			$this->lib  = new \PclZip( $zipname );
		}
	}

	/**
	 * Set data to Everest_Backup\Archiver::$temps.
	 *
	 * @param string $key Array key for the `Everest_Backup\Archiver::$temps`.
	 * @param mixed  $value Value to set according to the `$key`.
	 * @return void
	 * @since 1.0.0
	 */
	public function set_temp_data( $key, $value ) {
		$this->temps[ $key ] = $value;
	}

	/**
	 * Returns temp data according to the `$key`.
	 *
	 * @param string $key Array key for the `Everest_Backup\Archiver::$temps`.
	 * @return mixed
	 * @since 1.0.0
	 */
	public function get_temp_data( $key ) {
		if ( isset( $this->temps[ $key ] ) ) {
			return $this->temps[ $key ];
		}
	}

	/**
	 * Add the pass file size to total filesize.
	 *
	 * @param string $file Filename with path.
	 * @return void
	 * @since 1.0.0
	 */
	private function add_total_filesize( $file ) {

		if ( ! $file ) {
			return;
		}

		$filesize = filesize( $file );

		if ( is_int( $filesize ) ) {
			$this->total_files_size += $filesize;
		}

	}

	/**
	 * Returns total added file size.
	 *
	 * @return int
	 * @since 1.0.0
	 */
	public function get_total_filesize() {
		return $this->total_files_size;
	}

	/**
	 * Adds files to the list for the compression.
	 *
	 * @param string $file Filename with path.
	 * @return bool|void Return true on success.
	 * @since 1.0.0
	 */
	protected function add( $file ) {

		if ( ! is_file( $file ) ) {
			return;
		}

		$this->add_total_filesize( $file );

		$this->files[] = wp_normalize_path( $file );

		return true;
	}

	/**
	 * Returns an array of all the added files.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public function get_files() {
		return $this->files;
	}

	/**
	 * If ZipArchive class exists then handle compression using it.
	 *
	 * @return string Path to the zip with zipname.
	 * @since 1.0.0
	 */
	protected function zip_using_ziparchive() {

		$files      = $this->get_files();
		$ziparchive = $this->lib;
		$zipname    = $this->zipname;

		$ziparchive->open( $zipname, $ziparchive::CREATE );

		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {

				if ( everest_backup_has_aborted() ) {
					break;
				}

				$entryname = str_replace( $this->paths_to_replace, '', $file );

				$ziparchive->addFile( $file, $entryname );
			}
		}

		$compressed = $ziparchive->close();

		return $compressed && file_exists( $zipname ) ? $zipname : '';
	}

	/**
	 * If ZipArchive does not class exists then handle compression using pclzip.
	 *
	 * @return string Path to the zip with zipname.
	 * @since 1.0.0
	 */
	protected function zip_using_pclzip() {

		$files   = $this->get_files();
		$pclzip  = $this->lib;
		$zipname = $this->zipname;

		$compressed = array();
		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {
				if ( everest_backup_has_aborted() ) {
					break;
				}

				$entryname = str_replace( $this->paths_to_replace, '', $file );

				$list = $pclzip->add(
					array(
						array(
							PCLZIP_ATT_FILE_NAME          => $file,
							PCLZIP_ATT_FILE_NEW_FULL_NAME => $entryname,
						),
					)
				);

				if ( isset( $list[0] ) ) {
					$compressed[] = $list[0];
				}
			}
		}

		return $compressed && file_exists( $zipname ) ? $zipname : '';
	}
}
