<?php
/**
 * Handles the backup storage directory "ebwp-backups".
 *
 * @package everest-backup
 */

namespace Everest_Backup;

use Everest_Backup\Traits\Singleton;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Handles the backup storage directory "ebwp-backups".
 *
 * @since 1.0.0
 */
class Backup_Directory {

	use Singleton;

	/**
	 * Directory path to backups folder.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	private $backup_directory_path = EVEREST_BACKUP_BACKUP_DIR_PATH;

	/**
	 * Cached the backup files temporarily.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $cached_backups = array();

	/**
	 * File system class object.
	 *
	 * @var Filesystem
	 * @since 1.0.0
	 */
	private $filesystem;

	/**
	 * Init class.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {
		$this->filesystem = Filesystem::init();
	}

	/**
	 * Create backup directory and its required security files.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function create() {
		if ( $this->is_backup_dir_exits() ) {
			return;
		}
		$this->create_backup_dir();
		$this->create_security_files();
	}

	/**
	 * Returns true if backup directory is created and exists.
	 *
	 * @return boolean
	 * @since 1.0.0
	 */
	public function is_backup_dir_exits() {
		return $this->filesystem->is_dir( $this->backup_directory_path );
	}

	/**
	 * Create backup directory.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	private function create_backup_dir() {
		$this->filesystem->mkdir_p( $this->backup_directory_path );
	}

	/**
	 * Returns the array list of backups in ascending or descending order.
	 *
	 * @param string $order Accepts: `asc` or `desc`.
	 * @return array
	 */
	public function get_backups_by_order( $order = 'asc' ) {
		$backups = $this->get_backups( true );

		if ( ! $backups ) {
			return array();
		}

		$sorted = usort(
			$backups,
			function( $a, $b ) use ( $order ) {
				$time_a = filemtime( $a );
				$time_b = filemtime( $b );
				return 'asc' === $order ? strcmp( $time_b, $time_a ) : strcmp( $time_a, $time_b );
			}
		);

		return $sorted ? $backups : array();
	}

	/**
	 * Returns the path to the latest backup file in the backup directory.
	 *
	 * @return string
	 * @since 1.0.0
	 */
	public function get_latest_backup() {
		$backups = $this->get_backups_by_order();

		return ! empty( $backups[0] ) ? $backups[0] : '';
	}

	/**
	 * Returns the array of backup files older than `$days`.
	 *
	 * @param int $days Number of days.
	 * @return array
	 * @since 1.0.0
	 */
	public function get_backups_older_than( $days ) {

		if ( ! $days ) {
			return;
		}

		$now = time();

		$backups = $this->get_backups();

		$older_backups = array();

		if ( is_array( $backups ) && ! empty( $backups ) ) {
			foreach ( $backups as $backup ) {
				if ( empty( $backup['time'] ) ) {
					continue;
				}

				$diff = (int) abs( $backup['time'] - $now );

				$backup_days = round( $diff / DAY_IN_SECONDS );

				if ( $backup_days <= 1 ) {
					$backup_days = 1;
				}

				if ( $days >= $backup_days ) {
					continue;
				}

				$older_backups[] = $backup;
			}
		}

		return $older_backups;

	}

	/**
	 * List all the files from the backup directory.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public function get_files() {
		$files = $this->cached_backups;

		if ( ! $files ) {
			$files = $this->filesystem->list_files_all( $this->backup_directory_path );

			$this->cached_backups = $files;
		}

		return $files;
	}

	/**
	 * Returns the array of previous backup files from backup directory.
	 *
	 * @param bool $files_only If passed true then it returns array of files only.
	 * @return array
	 * @since 1.0.0
	 */
	public function get_backups( $files_only = false ) {

		$backups = array();

		/**
		 * Bail if backup directory is not created yet.
		 */
		if ( ! $this->is_backup_dir_exits() ) {
			return $backups;
		}

		$ext_type = EVEREST_BACKUP_BACKUP_FILE_EXTENSION;
		$files    = $this->get_files();

		/**
		 * Filter out non backup files.
		 */
		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {
				$ext = $this->filesystem->get_file_extension( $file );

				if ( ".{$ext}" !== $ext_type ) {
					continue;
				}

				$backups[] = $file;
			}
		}

		if ( ! $backups ) {
			return $backups;
		}

		if ( $files_only ) {
			return $backups;
		}

		$backups_data = array();

		if ( is_array( $backups ) && ! empty( $backups ) ) {
			foreach ( $backups as $backup ) {
				$backups_data[] = everest_backup_get_backup_file_info( $backup );
			}
		}

		return $backups_data;

	}

	/**
	 * Returns an array of files not related to backup directory.
	 *
	 * @param int $days As Days, pass 0 if you don't want to exclude latest files.
	 * @return array
	 * @since 1.0.0
	 */
	public function get_misc_files( $days = 0 ) {

		$misc_files = array();

		$ext_type = EVEREST_BACKUP_BACKUP_FILE_EXTENSION;
		$files    = $this->get_files();

		$security_files = array_keys( $this->get_security_files() );

		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {

				$fileinfo = pathinfo( $file );

				$basename  = $fileinfo['basename'];
				$extension = $fileinfo['extension'];

				// Ignore default security files.
				if ( in_array( $basename, $security_files, true ) ) {
					continue;
				}

				// Ignore backup file.
				if ( ".{$extension}" === $ext_type ) {
					continue;
				}

				$misc_files[] = $file;
			}
		}

		if ( ! $days ) {
			return $misc_files;
		}

		$now = time();

		$old_misc_files = array();

		if ( is_array( $misc_files ) && ! empty( $misc_files ) ) {
			foreach ( $misc_files as $misc_file ) {
				$backup = everest_backup_get_backup_file_info( $misc_file );

				if ( empty( $backup['time'] ) ) {
					continue;
				}

				$diff = (int) abs( $backup['time'] - $now );

				$backup_days = round( $diff / DAY_IN_SECONDS );

				if ( $backup_days <= 1 ) {
					$backup_days = 1;
				}

				if ( $days >= $backup_days ) {
					continue;
				}

				$old_misc_files[] = $backup;
			}
		}

		return $old_misc_files;

	}

	/**
	 * Returns array of security files with file contents.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	private function get_security_files() {
		$files_and_contents = array(
			'.htaccess'    => $this->htaccess_content(),
			'index.php'    => '<?php',
			'index.html'   => '',
			'log.json'     => '', // In this file, logs will be placed at the end of the process.
			'activity.txt' => '', // In this file, logs will be placed during the process.
		);

		return $files_and_contents;
	}

	/**
	 * Create security files inside backup directory folder.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	private function create_security_files() {

		$backup_directory_path = $this->backup_directory_path;
		$files_and_contents    = $this->get_security_files();

		if ( is_array( $files_and_contents ) && ! empty( $files_and_contents ) ) {
			foreach ( $files_and_contents as $filename => $content ) {

				$file = wp_normalize_path( $backup_directory_path . '/' . $filename );
				$this->filesystem->writefile( $file, $content );
			}
		}
	}

	/**
	 * Content for htaccess files.
	 *
	 * @return string
	 * @since 1.0.0
	 */
	private function htaccess_content() {
		return implode(
			PHP_EOL,
			array(
				'<IfModule mod_mime.c>',
				'AddType application/octet-stream ' . EVEREST_BACKUP_BACKUP_FILE_EXTENSION,
				'</IfModule>',
				'<IfModule mod_dir.c>',
				'DirectoryIndex index.php',
				'</IfModule>',
				'<IfModule mod_autoindex.c>',
				'Options -Indexes',
				'</IfModule>',
			)
		);
	}
}
