<?php
/**
 * Abstract class for handling cloud storage functionality.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Abstract class for handling cloud storage functionality.
 *
 * @abstract
 * @since 1.0.0
 */
class Cloud {

	/**
	 * Current cloud parameters that will be merged to package locations array.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $cloud_param = array();

	/**
	 * Settings > cloud fields names attributes prefix.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $fields_key = 'everest_backup_settings[cloud]';

	/**
	 * Init class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		$this->cloud_param = $this->set_cloud_param();
		add_filter( 'everest_backup_filter_package_locations', array( $this, 'package_locations' ) );
		add_action( 'everest_backup_settings_cloud_content', array( $this, 'render' ), 12, 2 );
	}

	/**
	 * Merge cloud parameters to package locations.
	 *
	 * @param array $package_locations Backup package locations.
	 * @return array Backup package locations.
	 * @since 1.0.0
	 */
	public function package_locations( $package_locations ) {
		if ( ! $this->cloud_param ) {
			return $package_locations;
		}

		$settings = everest_backup_get_settings();

		$cloud_keys = array_keys( $this->cloud_param );

		/**
		 * Automatically create cloud key for settings.
		 */
		if ( is_array( $cloud_keys ) && ! empty( $cloud_keys ) ) {
			foreach ( $cloud_keys as $cloud_key ) {
				if ( isset( $settings['cloud'][ $cloud_key ] ) ) {
					continue;
				}

				$settings['cloud'][ $cloud_key ] = array();

				everest_backup_update_settings( $settings );

			}
		}

		return array_merge( $package_locations, $this->cloud_param );
	}

	/**
	 * Set current cloud parameters that will be merged to package locations array.
	 *
	 * @abstract
	 * @return void
	 */
	protected function set_cloud_param() {
		_doing_it_wrong( __METHOD__, esc_html__( 'This method is supposed to be overridden by subclasses.', 'everest-backup' ), '' );
	}

	/**
	 * Returns fields name for the settings > cloud fields name attributes.
	 * Accepts values as:
	 * * $key = 'your_key';
	 * * $key = '[your_key][sub_key]';
	 *
	 * @param string $key Fields name attribute value.
	 * @return string Processed fields name attribute value for cloulds fields.
	 * @since 1.0.0
	 */
	protected function get_name( $key ) {
		return str_replace( array( '[[', ']]' ), array( '[', ']' ), "{$this->fields_key}[$key]" );
	}

	/**
	 * Render HTML in Settings > Cloud content.
	 *
	 * @param string $cloud_key Array key of cloud ( or package location ) paramaters passed to `everest_backup_filter_package_locations` filter hook.
	 * @param array  $settings Settings data.
	 * @abstract
	 * @return void
	 * @since 1.0.0
	 */
	public function render( $cloud_key, $settings ) { // @phpcs:ignore
		_doing_it_wrong( __METHOD__, esc_html__( 'This method is supposed to be overridden by subclasses.', 'everest-backup' ), '' );
	}
}
