<?php
/**
 * Class for creating zip files.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for creating zip files.
 *
 * @since 1.0.0
 */
class Compress extends Archiver {

	/**
	 * List of excluded files.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $excluded = array();

	/**
	 * Parameters to manipulate backup modules.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $params = array();

	/**
	 * Unique ID for the zip file.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	private $zip_uniqid;

	/**
	 * Current unix timestamp.
	 *
	 * @var int
	 * @since 1.0.0
	 */
	private $timestamp;

	/**
	 * Directory path to backups folder.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	private $backup_directory_path = EVEREST_BACKUP_BACKUP_DIR_PATH;

	/**
	 * Init class.
	 *
	 * @param array $params Parameters to manipulate backup modules.
	 * @since 1.0.0
	 */
	public function __construct( $params = array() ) {

		$this->timestamp  = time();
		$this->zip_uniqid = uniqid();

		$this->params = apply_filters( 'everest_backup_filter_backup_modules_params', $params );

		/**
		 * Create backup directory if it does not exists.
		 */
		Backup_Directory::init()->create();

		Logs::info( __( 'Initializing new archive.', 'everest-backup' ) );

		$zipname = $this->get_zipname();

		everest_backup_set_current_archive( $zipname );

		parent::__construct( $zipname );
	}

	/**
	 * Returns params.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public function get_params() {
		return $this->params;
	}

	/**
	 * Add path to list that needs to be replaced.
	 *
	 * @param string $path Path to add in the list.
	 * @return void
	 * @since 1.0.0
	 */
	public function add_paths_to_replace( $path ) {
		$this->paths_to_replace[] = $path;
	}

	/**
	 * It creates a dynamic zipname with full path without file extension.
	 * The zip file extension is being handled from parent class.
	 *
	 * @return string
	 * @since 1.0.0
	 */
	private function get_zipname() {
		$backup_directory_path = $this->backup_directory_path;

		$filename_block   = array();
		$filename_block[] = 'ebwp-';
		$filename_block[] = sanitize_title( preg_replace( '#^https?://#i', '', home_url() ) );
		$filename_block[] = '-' . $this->timestamp;
		$filename_block[] = '-' . $this->zip_uniqid;

		$filename = implode( '', $filename_block );

		$zipname = $backup_directory_path . '/' . $filename;

		return $zipname;

	}

	/**
	 * Add files to the list.
	 *
	 * @param array $files Array of files to add.
	 * @return array $files_add Array of files added to the lists, else empty array on failure.
	 *
	 * @since 1.0.0
	 */
	public function addfiles( $files ) {
		$files_add = array();

		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {

				if ( everest_backup_is_extension_excluded( $file ) ) {
					$this->excluded[] = $file;
					continue;
				}

				if ( $this->add( $file ) ) {
					$files_add[] = $file;
				}
			}
		}

		return $files_add;
	}

	/**
	 * Returns uniqid of current archive process.
	 *
	 * @return string Unique ID string.
	 * @since 1.0.0
	 */
	public function zip_uniqid() {
		return $this->zip_uniqid;
	}

	/**
	 * Returns current unix timestamp.
	 *
	 * @return int Unique ID string.
	 * @since 1.0.0
	 */
	public function timestamp() {
		return $this->timestamp;
	}

	/**
	 * Check disk space before archiving the files.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function prezip_disk_space_check() {

		Logs::info( __( 'Checking disk available space.', 'everest-backup' ) );

		$disk_free_space = disk_free_space( $this->backup_directory_path );
		$total_filesize  = $this->get_total_filesize();

		/* translators: %1$s is total filesize and %2$s is server available free disk space. */
		$message = sprintf( __( 'Required disk space: %1$s. Available disk space: %2$s', 'everest-backup' ), esc_html( size_format( $total_filesize ) ), esc_html( size_format( $disk_free_space ) ) );

		if ( $disk_free_space > $total_filesize ) {
			Logs::info( __( 'Disk space available.', 'everest-backup' ) );
			Logs::info( $message );
			return;
		}

		Logs::error( $message );
		everest_backup_send_error( $message );

	}

	/**
	 * Compress and zip the files and return zip path.
	 * This method must be call at the last, after all the files has been added to the list.
	 *
	 * @return string Full path to zip file with extension.
	 */
	public function zip() {
		$this->prezip_disk_space_check();

		$total_excluded = count( $this->excluded );

		Logs::info( $total_excluded . ' ' . _n( 'file excluded.', 'files excluded.', $total_excluded, 'everest-backup' ) );

		Logs::info( __( 'Wrapping things up.', 'everest-backup' ) );
		$zippath = 'ziparchive' === $this->type ? $this->zip_using_ziparchive() : $this->zip_using_pclzip();

		$zip = is_file( $zippath ) && 'ebwp' === pathinfo( $zippath, PATHINFO_EXTENSION ) ? $zippath : '';

		do_action( 'everest_backup_after_zip_done', $zip, $this );

		return $zip ? $zip : '';
	}

}
