<?php
/**
 * Database import/export core class.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Database import/export base class.
 *
 * @since 1.0.0
 */
class Database {

	/**
	 * WordPress database instance.
	 *
	 * @var \wpdb
	 */
	protected $wpdb = null;

	/**
	 * Full path to the sql file.
	 *
	 * @var string
	 * @since 1.0.2
	 */
	protected $filename = null;

	/**
	 * Database prefix for import/export.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $old_db_prefix = null;

	/**
	 * Database prefix for import/export.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected $new_db_prefix = null;

	/**
	 * Init class.
	 *
	 * @param \wpdb $wpdb WordPress database instance.
	 * @since 1.0.0
	 */
	public function __construct( $wpdb ) {
		$this->wpdb = $wpdb;
	}

	/**
	 * Modifies the database based on specified SQL statements.
	 *
	 * @param string $queries Database query.
	 * @since 1.0.0
	 */
	public function query( $queries ) {
		if ( ! $queries ) {
			return false;
		}

		$query = $this->wpdb->query( $queries ); // @phpcs:ignore

		$this->wpdb->flush();

		return $query;
	}

	/**
	 * Returns header for dump file.
	 *
	 * @return string
	 */
	public function get_header() {
		$wpdb = $this->wpdb;

		$dbhost = $wpdb->dbhost;
		$dbname = $wpdb->dbname;

		// Some info about software, source and time.
		$header = sprintf(
			"-- Everest Backup SQL Dump\n" .
			"--\n" .
			"-- Host: %s\n" .
			"-- Database: %s\n" .
			"-- Class: %s\n" .
			"--\n",
			$dbhost,
			$dbname,
			get_class( $this )
		);

		return $header;
	}

	/**
	 * Get database table names.
	 *
	 * @param null|string $custom_prefix If provided custom prefix then the tables array will have the `$custom_prefix` instead of actual `$wpdb` prefix.
	 * @return array
	 * @since 1.0.0
	 */
	public function get_tables( $custom_prefix = null ) {
		$wpdb = $this->wpdb;

		$prefix = $wpdb->prefix;
		$tables = $wpdb->get_results( 'SHOW TABLES', ARRAY_A ); // @phpcs:ignore

		$table_names = is_array( $tables ) ? array_map(
			function( $table ) use ( $custom_prefix, $prefix ) {
				$table_name = array_values( $table );
				if ( isset( $table_name[0] ) ) {
					if ( $custom_prefix ) {
						return everest_backup_str_replace_once( $prefix, $custom_prefix, $table_name[0] );
					}
					return $table_name[0];
				}
			},
			$tables
		) : array();

		$wpdb->flush();

		return $table_names;
	}

	/**
	 * Get the CREATE TABLE sql for available tables.
	 *
	 * @param string $table_name Name of database table.
	 * @return string CREATE TABLE sql query.
	 * @since 1.0.0
	 */
	public function get_create_table( $table_name ) {
		$wpdb = $this->wpdb;

		$results = $wpdb->get_results( $wpdb->prepare( 'SHOW CREATE TABLE %1s;', $table_name ), ARRAY_N );  // @phpcs:ignore

		$wpdb->flush();

		return ! empty( $results[0][1] ) ? $results[0][1] . ';' : '';
	}

	/**
	 * Replace list of users login related meta key or values that needs to be changed according to the current table name.
	 *
	 * @param string $input SQL input string.
	 * @return string
	 * @since 1.0.0
	 */
	public function replace_users_login_related_metas( $input ) {

		if ( ! $input ) {
			return $input;
		}

		$lists = array();
		$metas = array(
			'user_roles',
			'capabilities',
			'user_level',
			'dashboard_quick_press_last_post_id',
		);

		if ( is_array( $metas ) && ! empty( $metas ) ) {
			foreach ( $metas as $meta ) {
				$lists['old'][] = $this->old_db_prefix . $meta;
				$lists['new'][] = $this->new_db_prefix . $meta;
			}
		}

		$replaced = str_replace( $lists['old'], $lists['new'], $input );

		return $replaced && is_string( $replaced ) ? $replaced : $input;

	}

	/**
	 * Converts table array data into "INSERT INTO" sql query.
	 *
	 * @param string $table_name Table data.
	 * @return array INSERT INTO sql query.
	 * @since 1.0.0
	 */
	public function get_insert_into( $table_name ) {

		if ( ! $table_name ) {
			return;
		}

		$table_data = $this->get_table_data( $table_name );

		if ( ! $table_data ) {
			return;
		}

		$column_types = $this->get_column_types( $table_name );

		$insert_queries = array();

		if ( is_array( $table_data ) && ! empty( $table_data ) ) {

			$insert_queries[] = "INSERT INTO `$table_name` VALUES\n";

			foreach ( $table_data as $data ) {
				$table_values = array();

				if ( is_array( $data ) && ! empty( $data ) ) {
					foreach ( $data as $column => $value ) {
						$fix_serialize  = $this->fix_str_length( $value ); // @since 1.0.2
						$table_values[] = $this->prepare_table_values( $fix_serialize, $column_types[ $column ] );
					}
				}

				if ( is_array( $table_values ) && ! empty( $table_values ) ) {
					$last_index = ( count( $table_values ) - 1 );
					foreach ( $table_values as $index => $table_value ) {

						if ( 0 === $index ) {
							$insert_queries[] = "({$table_value}, ";
						}

						if ( ! ( 0 === $index ) && ! ( $index === $last_index ) ) {
							$insert_queries[] = "{$table_value}, ";
						}

						if ( ( $index === $last_index ) ) {
							$insert_queries[] = "{$table_value}),\n";
						}
					}
				}

			}
		}

		unset( $table_data );
		unset( $column_types );

		if ( ! $insert_queries ) {
			return;
		}

		$query_count = count( $insert_queries );

		$last_key = $query_count ? ( $query_count - 1 ) : 0;

		$insert_queries[ $last_key ] = rtrim( $insert_queries[ $last_key ], "),\n" ) . ");\n\n";

		return $insert_queries;

	}

	/**
	 * Dumps sql file.
	 *
	 * @return bool
	 * @since 1.0.2
	 */
	protected function dump_sql() {

		$tables = $this->get_tables();

		if ( is_array( $tables ) && ! empty( $tables ) ) {

			$created = $this->write( $this->get_header() );

			foreach ( $tables as $table_name ) {

				$new_table_name = everest_backup_str_replace_once( $this->old_db_prefix, $this->new_db_prefix, $table_name );

				$created = $this->write( "\nDROP TABLE IF EXISTS `{$new_table_name}`;\n" );

				$create_table = $this->get_create_table( $table_name );

				$create_table = $this->replace_table_name( $create_table, $table_name, $new_table_name );

				$create_table = $this->replace_table_comments( $create_table );

				$create_table = $this->replace_table_constraints( $create_table );

				$create_table = $this->replace_table_options( $create_table );

				$created = $this->write( "$create_table\n\n" );

				unset( $create_table );

				$insert_queries = $this->get_insert_into( $table_name );

				if ( is_array( $insert_queries ) && ! empty( $insert_queries ) ) {

					$created = $this->write( "\nSTART TRANSACTION;\n" );

					foreach ( $insert_queries as $insert_query ) {

						$insert = $this->replace_table_name( $insert_query, $table_name, $new_table_name );
						$insert = $this->replace_users_login_related_metas( $insert );

						$created = $this->write( $insert );
					}

					$created = $this->write( "COMMIT;\n" );

				}

				unset( $insert_queries );

			}
		}

		return isset( $created ) && is_int( $created );

	}

	/**
	 * Fix string length in serialize value.
	 *
	 * @param string $serialized_string
	 * @return mixed
	 * @since 1.0.2
	 */
	protected function fix_str_length( $serialized_string ) {

		if ( ! $serialized_string ) {
			return $serialized_string;
		}

		if ( ! is_serialized( $serialized_string ) ) {
			return $serialized_string;
		}

		// At first, check if "fixing" is really needed at all. After that, security checkup.
		if ( preg_match('/^[aOs]:/', $serialized_string ) ) {
			$serialized_string = preg_replace_callback(
				'/s\:(\d+)\:\"(.*?)\";/s',
				function( $matches ) {
					return 's:' . strlen( $matches[2] ) . ':"' . $matches[2] . '";';
				},
				$serialized_string
			);
		}

		return $serialized_string;

	}

	/**
	 * Escape MySQL special characters
	 *
	 * @param  string $data Data to escape.
	 * @return string
	 */
	public function escape_mysql( $data ) {
		return strtr(
			$data,
			array_combine(
				array( "\x00", "\n", "\r", '\\', "'", '"', "\x1a" ),
				array( '\\0', '\\n', '\\r', '\\\\', "\\'", '\\"', '\\Z' )
			)
		);
	}

	/**
	 * Unescape MySQL special characters
	 *
	 * @param  string $data Data to unescape.
	 * @return string
	 */
	public function unescape_mysql( $data ) {
		return strtr(
			$data,
			array_combine(
				array( '\\0', '\\n', '\\r', '\\\\', "\\'", '\\"', '\\Z' ),
				array( "\x00", "\n", "\r", '\\', "'", '"', "\x1a" )
			)
		);
	}

	/**
	 * Write data to sql file.
	 *
	 * @param string $data
	 * @return int|false False on failure.
	 * @since 1.0.2
	 */
	public function write( $data ) {

		if ( ! $this->filename ) {
			return;
		}

		$write = Filesystem::init()->writefile( $this->filename, $data, true );

		return $write ? 1 : false;

	}

	/**
	 * Get table data.
	 *
	 * @param string $table_name Name of database table.
	 * @return array Table data.
	 * @since 1.0.0
	 */
	protected function get_table_data( $table_name ) {
		$wpdb = $this->wpdb;

		$prefix = $wpdb->prefix;

		$is_options_table = "{$prefix}options" === $table_name;

		if ( $is_options_table ) {
			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %1s WHERE option_name NOT LIKE "%_transient_%" AND option_name NOT IN (%s, %s)', $table_name, EVEREST_BACKUP_LOGS_KEY, EVEREST_BACKUP_SETTINGS_KEY ), ARRAY_A );  // @phpcs:ignore
		} else {
			$results = $wpdb->get_results( $wpdb->prepare( 'SELECT * FROM %1s', $table_name ), ARRAY_A );  // @phpcs:ignore
		}

		$wpdb->flush();

		return ! empty( $results ) ? $results : array();
	}

	/**
	 * Returns columns data types of a table.
	 *
	 * @param string $table_name Name of database table.
	 * @return array
	 * @since 1.0.0
	 */
	protected function get_column_types( $table_name ) {
		$wpdb = $this->wpdb;

		$column_types = $wpdb->get_results( $wpdb->prepare( 'SHOW COLUMNS FROM %1s;', $table_name ), ARRAY_A );  // @phpcs:ignore

		$wpdb->flush();

		return is_array( $column_types ) && ! empty( $column_types ) ? wp_list_pluck( $column_types, 'Type', 'Field' ) : array();

	}

	/**
	 * Check if current string is just a sql comment `--` or valid string.
	 *
	 * @param string $input SQL input string.
	 * @return boolean
	 * @since 1.0.0
	 */
	protected function is_valid( $input ) {
		return ! ( ( '--' === substr( $input, 0, 2 ) ) || ( '' === $input ) );
	}

	/**
	 * Check if current input string is the end of sql query.
	 *
	 * @param string $input SQL input string.
	 * @return boolean
	 * @since 1.0.0
	 */
	protected function is_query_end( $input ) {
		return ( ';' === substr( trim( $input ), -1, 1 ) );
	}

	/**
	 * Prepare table values
	 *
	 * @param  string  $input       Table value.
	 * @param  integer $column_type Column type.
	 * @return string
	 */
	protected function prepare_table_values( $input, $column_type ) {
		if ( is_null( $input ) ) {
			return 'NULL';
		} elseif ( stripos( $column_type, 'tinyint' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'smallint' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'mediumint' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'int' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'bigint' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'float' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'double' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'decimal' ) === 0 ) {
			return $input;
		} elseif ( stripos( $column_type, 'bit' ) === 0 ) {
			return $input;
		}

		return "'" . $this->escape_mysql( $input ) . "'";
	}

	/**
	 * Rename old table name into new.
	 *
	 * @param string $input SQL statement.
	 * @param array  $old_tables Old tables array with old original prefix.
	 * @return string
	 * @since 1.0.0
	 */
	protected function rename_table_names( $input, $old_tables ) {
		$old_prefix = $this->old_db_prefix;
		$new_prefix = $this->new_db_prefix;

		$new_tables = str_replace( $old_prefix, $new_prefix, implode( ',', $old_tables ) );

		return str_replace( $old_tables, explode( ',', $new_tables ), $input );

	}

	/**
	 * Replace table name
	 *
	 * @param  string $input          Table value
	 * @param  string $old_table_name Old table name
	 * @param  string $new_table_name New table name
	 * @return string
	 */
	protected function replace_table_name( $input, $old_table_name, $new_table_name ) {
		$position = stripos( $input, "`$old_table_name`" );
		if ( $position !== false ) {
			$input = substr_replace( $input, "`$new_table_name`", $position, strlen( "`$old_table_name`" ) );
		}

		return $input;
	}


	/**
	 * Replace table comments
	 *
	 * @param  string $input SQL statement.
	 * @return string
	 */
	protected function replace_table_comments( $input ) {
		return preg_replace( '/\/\*(.+?)\*\//s', '', $input );
	}

	/**
	 * Replace table constraints
	 *
	 * @param  string $input SQL statement.
	 * @return string
	 */
	protected function replace_table_constraints( $input ) {
		$pattern = array(
			'/\s+CONSTRAINT(.+)REFERENCES(.+),/i',
			'/,\s+CONSTRAINT(.+)REFERENCES(.+)/i',
		);

		return preg_replace( $pattern, '', $input );
	}

	/**
	 * Replace table options
	 *
	 * @param  string $input SQL statement.
	 * @return string
	 */
	protected function replace_table_options( $input ) {
		$search  = array(
			'TYPE=InnoDB',
			'TYPE=MyISAM',
			'ENGINE=Aria',
			'TRANSACTIONAL=0',
			'TRANSACTIONAL=1',
			'PAGE_CHECKSUM=0',
			'PAGE_CHECKSUM=1',
			'TABLE_CHECKSUM=0',
			'TABLE_CHECKSUM=1',
			'ROW_FORMAT=PAGE',
			'ROW_FORMAT=FIXED',
			'ROW_FORMAT=DYNAMIC',
		);
		$replace = array(
			'ENGINE=InnoDB',
			'ENGINE=MyISAM',
			'ENGINE=MyISAM',
			'',
			'',
			'',
			'',
			'',
			'',
			'',
			'',
			'',
		);

		return str_ireplace( $search, $replace, $input );
	}

}

