<?php
/**
 * Main class that initialize everything.
 *
 * @package everest-backup
 */

use Everest_Backup\Backup_Directory;
use Everest_Backup\Logs;
use Everest_Backup\Modules\Cloner;
use Everest_Backup\Modules\Cron_Handler;
use Everest_Backup\Traits\Singleton;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'Everest_Backup' ) ) {

	/**
	 * Main class that initialize everything.
	 *
	 * @since 1.0.0
	 */
	class Everest_Backup {

		use Singleton;

		/**
		 * Init class.
		 *
		 * @since 1.0.0
		 */
		private function __construct() {
			do_action( 'everest_backup_init', $this );

			register_activation_hook( EVEREST_BACKUP_FILE, array( $this, 'on_activation' ) );
			register_deactivation_hook( EVEREST_BACKUP_FILE, array( $this, 'on_deactivation' ) );
			$this->init_hooks();

			do_action( 'everest_backup_loaded', $this );
		}

		/**
		 * Initialize hooks.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		private function init_hooks() {
			add_action( 'admin_init', array( $this, 'on_admin_init' ) );
			add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts' ) );
		}

		/**
		 * On plugin activation.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		public function on_activation() {
			Backup_Directory::init()->create();
		}

		/**
		 * On plugin deactivation.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		public function on_deactivation() {
			$cron_handler = new Cron_Handler();
			$cron_handler->unschedule_events();
		}

		/**
		 * On admin_init hooks.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		public function on_admin_init() {
			$everest_backup_cloner = new Cloner();
			$everest_backup_cloner->handle_package_clone();

			$this->activate_addon();
			$this->save_settings();
			$this->remove_backup_file();
			$this->restore_rollback();
			$this->bulk_remove_logs();
		}

		/**
		 * Activate the selected addon if it is submitted from the Everest backup addon page.
		 *
		 * @return void
		 */
		private function activate_addon() {
			$data = everest_backup_get_submitted_data( 'post' );

			if ( empty( $data['page'] ) ) {
				return;
			}

			if ( 'everest-backup-addons' !== $data['page'] ) {
				return;
			}

			if ( empty( $data['plugin'] ) ) {
				everest_backup_set_notice( __( 'Plugin slug empty.', 'everest-backup' ), 'notice-error' );
				return;
			}

			$activate = activate_plugin( $data['plugin'] );

			if ( ! is_wp_error( $activate ) ) {
				everest_backup_set_notice( __( 'Addon activated.', 'everest-backup' ), 'notice-success' );
			} else {
				$err_msg = $activate->get_error_message();
				everest_backup_set_notice( $err_msg, 'notice-error' );
			}
		}

		/**
		 * Save settings data.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		private function save_settings() {
			$settings_data = everest_backup_get_submitted_data( 'post' );

			$submitted_data = ! empty( $settings_data[ EVEREST_BACKUP_SETTINGS_KEY ] ) ? $settings_data[ EVEREST_BACKUP_SETTINGS_KEY ] : array();

			if ( ! $submitted_data ) {
				return;
			}

			if ( ! everest_backup_verify_nonce( EVEREST_BACKUP_SETTINGS_KEY . '_nonce' ) ) {
				everest_backup_set_notice( __( 'Nonce verification failed.', 'everest-backup' ), 'notice-error' );
				return;
			}

			$saved_settings = everest_backup_get_settings();

			$settings = array_merge( $saved_settings, $submitted_data );

			do_action( 'everest_backup_before_settings_save', $settings );

			everest_backup_update_settings( $settings );

			do_action( 'everest_backup_after_settings_save', $settings );

			everest_backup_set_notice( __( 'Settings saved.', 'everest-backup' ), 'notice-success' );
		}

		/**
		 * Remove backup file.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		private function remove_backup_file() {
			$get = everest_backup_get_submitted_data( 'get' );

			$page = ! empty( $get['page'] ) ? $get['page'] : '';

			if ( 'everest-backup-history' !== $page ) {
				return;
			}

			$bulk_action = isset( $get['action2'] ) ? $get['action2'] : '';
			$cloud       = isset( $get['cloud'] ) ? $get['cloud'] : 'server';

			if ( '-1' === $bulk_action ) {
				return;
			}

			if ( 'server' !== $cloud ) {
				return do_action( 'everest_backup_override_file_remove', $get );
			}

			$history_page_url = network_admin_url( "/admin.php?page={$page}" );

			$backup_dir = EVEREST_BACKUP_BACKUP_DIR_PATH;

			if ( $bulk_action ) {
				/**
				 * If we are here, we are removing files in bulk.
				 */

				$files = isset( $get['remove'] ) ? $get['remove'] : '';

				if ( is_array( $files ) && ! empty( $files ) ) {
					foreach ( $files as $file ) {
						$file_path = $backup_dir . DIRECTORY_SEPARATOR . $file;

						if ( ! is_file( $file_path ) ) {
							continue;
						}

						unlink( $file_path );
					}
				}
			} else {

				$action = ! empty( $get['action'] ) ? $get['action'] : '';
				$file   = ! empty( $get['file'] ) ? $get['file'] : '';

				if ( 'remove' !== $action && empty( $file ) ) {
					return;
				}

				$file_path = $backup_dir . DIRECTORY_SEPARATOR . $file;

				if ( ! is_file( $file_path ) ) {
					everest_backup_set_notice(
						'<strong>' . $file . '</strong> ' . __( 'does not exists.', 'everest-backup' ),
						'notice-error'
					);

					$redirect = remove_query_arg( array( 'action', 'file' ), $history_page_url );
					if ( wp_safe_redirect( $redirect ) ) {
						exit;
					}

					return;
				}

				if ( unlink( $file_path ) ) {
					everest_backup_set_notice(
						'<strong>' . $file . '</strong> ' . __( 'successfully removed from the server.', 'everest-backup' ),
						'notice-success'
					);

					$redirect = remove_query_arg( array( 'action', 'file' ), $history_page_url );
					if ( wp_safe_redirect( $redirect ) ) {
						exit;
					}

					return;
				}

				everest_backup_set_notice(
					__( 'Unable to remove file', 'everest-backup' ) . ' <strong>' . $file . '</strong>',
					'notice-error'
				);
			}

			$redirect = remove_query_arg( array( 'action', 'file' ), $history_page_url );
			if ( wp_safe_redirect( $redirect ) ) {
				exit;
			}
		}

		/**
		 * Roll back to the previous selected backup version.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		private function restore_rollback() {
			$get = everest_backup_get_submitted_data( 'get' );

			$page     = ! empty( $get['page'] ) ? $get['page'] : '';
			$action   = ! empty( $get['action'] ) ? $get['action'] : '';
			$rollback = ! empty( $get['rollback'] ) ? $get['rollback'] : '';

			if ( 'everest-backup-import' !== $page ) {
				return;
			}

			if ( 'rollback' !== $action ) {
				return;
			}

			define( 'EVEREST_BACKUP_DOING_ROLLBACK', true );

			if ( ! $rollback ) {
				return;
			}

			do_action( 'wp_ajax_everest_backup_import' ); // @phpcs:ignore

			if ( wp_safe_redirect( network_admin_url( '/admin.php?page=everest-backup-logs' ) ) ) {
				exit;
			}
		}

		/**
		 * Remove logs from the database.
		 *
		 * @return void
		 * @since 1.0.0
		 */
		private function bulk_remove_logs() {
			$get = everest_backup_get_submitted_data( 'get' );

			$page = ! empty( $get['page'] ) ? $get['page'] : '';

			if ( 'everest-backup-logs' !== $page ) {
				return;
			}

			if ( isset( $get['clear_all_logs'] ) ) {
				return Logs::delete_all_logs();
			}

			$bulk_action = isset( $get['action2'] ) ? $get['action2'] : '';

			if ( 'remove' === $bulk_action ) {
				$keys = isset( $get['remove'] ) ? $get['remove'] : '';

				Logs::delete( $keys );
			}

		}

		/**
		 * Args for the pluploader.
		 *
		 * @return array
		 * @since 1.0.0
		 */
		private function plupload_args() {

			$nonce = everest_backup_create_nonce( 'everest_backup_ajax_nonce' );

			$url = admin_url( "/admin-ajax.php?action=everest_backup_import&everest_backup_ajax_nonce={$nonce}" );

			return array(
				'runtimes'         => 'html5',
				'browse_button'    => 'plupload-browse-button',
				'container'        => 'plupload-upload-ui',
				'drop_element'     => 'drag-drop-area',
				'file_data_name'   => 'file',
				'multiple_queues'  => false,
				'multi_selection'  => false,
				'url'              => $url,
				'filters'          => array(
					'mime_types' => array(
						array(
							'title'      => __( 'EBWP File', 'everest-backup' ),
							'extensions' => str_replace( '.', '', EVEREST_BACKUP_BACKUP_FILE_EXTENSION ),
						),
					),
				),
				'multipart'        => true,
				'urlstream_upload' => true,
			);

		}

		/**
		 * Localized data.
		 */
		private function localized_data() {

			$max_upload_size = everest_backup_max_upload_size();

			$data = array(
				'_nonce'        => everest_backup_create_nonce( 'everest_backup_ajax_nonce' ),
				'ajaxUrl'       => admin_url( '/admin-ajax.php' ),
				'doingRollback' => everest_backup_doing_rollback(),
				'maxUploadSize' => $max_upload_size,
				'fileExtension' => str_replace( '.', '', EVEREST_BACKUP_BACKUP_FILE_EXTENSION ),
				'pluploadArgs'  => $this->plupload_args(),
				'locale'        => array(
					/* translators: Here, %s is the size limit set by the server. */
					'fileSizeExceedMessage' => sprintf( __( 'The file size is larger than %s', 'everest-backup' ), size_format( $max_upload_size ) ),
					'zipDownloadBtn'        => __( 'Download File', 'everest-backup' ),
					'migrationPageBtn'      => __( 'Generate Migration Key', 'everest-backup' ),
					'backupMessage'         => __( 'Please wait while we are doing the backup. You will get a detailed log after the backup is completed.', 'everest-backup' ),
					'restoreMessage'        => __( 'Restoration is in progress, please do not close this tab or window.', 'everest-backup' ),
					'abortAlert'            => __( 'Are you sure you want to stop this backup process?', 'everest-backup' ),
					'abortMessage'          => __( 'Please wait while we are removing the archived files.', 'everest-backup' ),
				),
				'adminPages'    => array(
					'dashboard' => network_admin_url(),
					'backup'    => network_admin_url( 'admin.php?page=everest-backup-export' ),
					'import'    => network_admin_url( '/admin.php?page=everest-backup-import' ),
					'history'   => network_admin_url( '/admin.php?page=everest-backup-history' ),
					'logs'      => network_admin_url( '/admin.php?page=everest-backup-logs' ),
					'settings'  => network_admin_url( '/admin.php?page=everest-backup-settings' ),
				),
			);

			return apply_filters( 'everest_backup_filter_localized_data', $data );
		}

		/**
		 * Load admin scripts.
		 *
		 * @param string $hook Current page slug id.
		 */
		public function admin_scripts( $hook ) {

			if ( false === strstr( $hook, 'everest-backup' ) ) {
				return;
			}

			$version = time(); // To tackle issues caused by cache plugins.

			wp_enqueue_style( 'everest-backup-admin-styles', EVEREST_BACKUP_URL . 'assets/css/admin.css', array(), $version, 'all' );

			switch ( $hook ) {
				case 'toplevel_page_everest-backup-export':
					$filetype = 'backup';
					break;

				case 'everest-backup_page_everest-backup-import':
					wp_enqueue_script( 'plupload-all' );

					$filetype = 'restore';
					break;

				case 'everest-backup_page_everest-backup-migration_clone':
					$filetype = 'migration-clone';
					break;

				case 'everest-backup_page_everest-backup-settings':
					$filetype = 'settings';
					break;

				case 'everest-backup_page_everest-backup-addons':
					$filetype = 'addons';
					break;

				default:
					$filetype = '';
					break;
			}

			if ( ! $filetype ) {
				return;
			}

			if ( 'backup' === $filetype || 'restore' === $filetype || 'migration-clone' === $filetype ) {

				// We don't want heartbeat to occur when importing/exporting.
				wp_deregister_script( 'heartbeat' );

				// We don't want auth check for monitoring whether the user is still logged in.
				remove_action( 'admin_enqueue_scripts', 'wp_auth_check_load' );

			}

			$handle   = "everest-backup-{$filetype}-script";
			$filepath = "assets/js/{$filetype}.js";

			$localized_data = $this->localized_data();

			wp_register_script( $handle, EVEREST_BACKUP_URL . $filepath, array(), $version, true );

			wp_localize_script( $handle, '_everest_backup', $localized_data );

			wp_enqueue_script( $handle );
		}

	}
}
