<?php
/**
 * Logs the process to the session.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class to log the process to the session.
 *
 * @since 1.0.0
 */
class Logs {

	/**
	 * Whether or not logging is closed.
	 *
	 * @var boolean
	 * @since 1.0.0
	 */
	protected static $is_closed = false;

	/**
	 * Temporary process logs.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected static $logs = array();

	/**
	 * Option key for the option table.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	protected static $logs_key = EVEREST_BACKUP_LOGS_KEY;

	/**
	 * Returns process type from logs array.
	 *
	 * @param array $logs Logs array.
	 * @return array
	 * @since 1.0.0
	 */
	public static function get_process_type( $logs ) {

		$process_types = everest_backup_get_process_types();

		$key = everest_backup_array_search( $logs, 'init', array_keys( $process_types ) );

		if ( ! empty( $logs[ $key ]['init'] ) ) {
			return array(
				'key'  => $key,
				'type' => $logs[ $key ]['init'],
			);
		}

		$key = everest_backup_array_search( $logs, 'type', array_keys( $process_types ) );

		if ( ! empty( $logs[ $key ]['type'] ) ) {
			return array(
				'key'  => $key,
				'type' => $logs[ $key ]['type'],
			);
		}

	}

	/**
	 * Save collected logs in the option table. This method must be called before `Everest_Backup\Logs::clear` method.
	 *
	 * @database
	 * @return void
	 * @since 1.0.0
	 */
	public static function save() {

		$time = time();
		$logs = self::retrive();

		$temp_logs = self::get();

		if ( ! $temp_logs ) {
			return;
		}

		$process_types = self::get_process_type( $temp_logs );

		$key = ! empty( $process_types['key'] ) ? $process_types['key'] : '';

		$type = ! empty( $process_types['type'] ) ? $process_types['type'] : '';

		$logs[ $time ] = array();

		$logs[ $time ]['time'] = $time;
		$logs[ $time ]['type'] = $type;

		if ( $type ) {
			unset( $temp_logs[ $key ] );
		}

		$logs[ $time ]['logs'] = array_values( $temp_logs );

		do_action( 'everest_backup_before_logs_save', $temp_logs, $logs );

		update_option( self::$logs_key, $logs );

		do_action( 'everest_backup_after_logs_save', $temp_logs, $logs );

	}

	/**
	 * Appends content to the activity.txt file in backup directory.
	 *
	 * @param string|array $data Data to save in activity.txt file.
	 * @param bool         $init Whether or not if it is first init.
	 * @return bool
	 */
	public static function save_to_activity_log( $data, $init = false ) {

		$activity_log_file = wp_normalize_path( EVEREST_BACKUP_BACKUP_DIR_PATH . '/activity.txt' );

		$content = '';

		if ( $init ) {

			global $wp_version, $wp_db_version;

			if ( file_exists( $activity_log_file ) ) {

				/**
				 * Create new file during every new instance of the process.
				 */
				unlink( $activity_log_file );
			}

			$content .= '============================================';
			$content .= PHP_EOL;
			$content .= 'Server Engine: ' . ( ! empty( $_SERVER['SERVER_SOFTWARE'] ) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) ) : 'N/A' );
			$content .= PHP_EOL;
			$content .= 'PHP Version: ' . PHP_VERSION;
			$content .= PHP_EOL;
			$content .= 'DB Version: ' . $wp_db_version;
			$content .= PHP_EOL;
			$content .= 'WP Version: ' . $wp_version;
			$content .= PHP_EOL;
			$content .= 'EB Version: ' . EVEREST_BACKUP_VERSION;
			$content .= PHP_EOL;
			$content .= 'Multisite: ' . is_multisite();
			$content .= PHP_EOL;
			$content .= 'Localhost: ' . everest_backup_is_localhost();
			$content .= PHP_EOL;
			$content .= 'EB Debug: ' . everest_backup_is_debug_on();
			$content .= PHP_EOL;
			$content .= 'WP Debug: ' . ( defined( 'WP_DEBUG' ) && WP_DEBUG );
			$content .= PHP_EOL;
			$content .= 'WP Debug Log: ' . ( defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG );
			$content .= PHP_EOL;
			$content .= 'Timestamp: ' . time();
			$content .= PHP_EOL;
			$content .= 'Timezone: ' . wp_date( 'e' );
			$content .= PHP_EOL;
			$content .= 'Active Addons: ' . wp_json_encode( everest_backup_installed_addons( 'active' ) );
			$content .= PHP_EOL;
			$content .= '============================================';
			$content .= PHP_EOL;
		}

		$content .= '[' . wp_date( 'd-M-Y h:i:s' ) . '] ';
		$content .= is_array( $data ) ? wp_json_encode( $data ) : $data;
		$content .= PHP_EOL;

		return Filesystem::init()->writefile( $activity_log_file, $content, true );

	}

	/**
	 * Save logs to log.json file in backup directory.
	 *
	 * @return void
	 */
	public static function save_to_log_json() {

		$time = time();
		$logs = array();

		$temp_logs = self::get();

		if ( ! $temp_logs ) {
			return;
		}

		$process_types = self::get_process_type( $temp_logs );

		$key = ! empty( $process_types['key'] ) ? $process_types['key'] : '';

		$type = ! empty( $process_types['type'] ) ? $process_types['type'] : '';

		$logs[ $time ] = array();

		$logs[ $time ]['time'] = $time;
		$logs[ $time ]['type'] = $type;

		if ( $type ) {
			unset( $temp_logs[ $key ] );
		}

		$logs[ $time ]['logs'] = array_values( $temp_logs );

		$log_file_path = wp_normalize_path( EVEREST_BACKUP_BACKUP_DIR_PATH . '/log.json' );

		if ( file_exists( $log_file_path ) ) {
			unlink( $log_file_path );
		}

		return Filesystem::init()->writefile( $log_file_path, wp_json_encode( $logs ) );

	}

	/**
	 * Retrive logs from the option table.
	 *
	 * @database
	 * @return array
	 * @since 1.0.0
	 */
	public static function retrive() {
		return (array) get_option( self::$logs_key, array() );
	}

	/**
	 * Delete/unset logs from the option table.
	 *
	 * @database
	 * @param int[] $keys Array of logs keys.
	 * @return void
	 * @since 1.0.0
	 */
	public static function delete( $keys ) {
		if ( ! $keys ) {
			return;
		}

		$logs = self::retrive();

		if ( is_array( $keys ) && ! empty( $keys ) ) {
			foreach ( $keys as $key ) {
				if ( isset( $logs[ $key ] ) ) {
					unset( $logs[ $key ] );
				}
			}
		}

		update_option( self::$logs_key, $logs );
	}

	/**
	 * Delete all logs from the database ( option table ).
	 *
	 * @database
	 * @return bool
	 * @since 1.0.1
	 */
	public static function delete_all_logs() {
		return delete_option( self::$logs_key );
	}

	/**
	 * Set logs log.
	 *
	 * @param array $data Data to save for logs..
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public static function set( $data ) {

		self::save_to_activity_log( $data );

		/**
		 * Stop setting logs if logging is closed.
		 * This is to prevent "SyntaxError: JSON.parse: unexpected end of data ajax response" type of errors in client side.
		 */
		if ( self::$is_closed ) {
			return;
		}

		if ( isset( $data['type'] ) && 'error' === $data['type'] ) {
			if ( is_int( strpos( $data['message'], WP_PLUGIN_DIR ) ) && false === strpos( $data['message'], EVEREST_BACKUP_PATH ) ) {

				/**
				 * Do not log if error is thrown from other plugins except Everest Backup.
				 */
				return;
			}
		}

		if ( ! session_id() ) {
			session_start();
		}

		if ( ! isset( $_SESSION['ebwp_logs'] ) ) {
			$_SESSION['ebwp_logs'] = array();
		}

		$_SESSION['ebwp_logs'][] = $data;

	}

	/**
	 * Get logs log.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public static function get() {

		if ( ! session_id() ) {
			session_start();
		}

		$sorted = array();
		$logs   = isset( $_SESSION['ebwp_logs'] ) ? everest_backup_sanitize_array( $_SESSION['ebwp_logs'] ) : array();
		$key    = everest_backup_array_search( $logs, 'type', 'done' );

		/**
		 * Make sure, done is always at the end of the array.
		 */
		if ( is_int( $key ) ) {
			$done = $logs[ $key ];

			unset( $logs[ $key ] );

			$sorted = array_values( $logs );

			$sorted[] = $done;
		}

		return $sorted ? $sorted : $logs;
	}

	/**
	 * Set process log type. To add other process types, @see `everest_backup_get_process_types()`
	 *
	 * @param string $type Current process type for log.
	 * @return void
	 * @since 1.0.0
	 */
	public static function init( $type ) {
		static $called = 0;

		/**
		 * Do not call this method more than one time in a single process run.
		 */
		if ( $called ) {
			return;
		}

		self::save_to_activity_log( '', true );

		self::set(
			array(
				'init' => $type,
			)
		);

		$called++;
	}

	/**
	 * Set info log.
	 *
	 * @param string $message Info message.
	 * @return void
	 * @since 1.0.0
	 */
	public static function info( $message ) {
		self::set(
			array(
				'type'    => 'info',
				'message' => $message,
			)
		);
	}

	/**
	 * Set warning log.
	 *
	 * @param string $message Warning message.
	 * @return void
	 * @since 1.0.0
	 */
	public static function warn( $message ) {
		self::set(
			array(
				'type'    => 'warning',
				'message' => $message,
			)
		);
	}

	/**
	 * Set error log.
	 *
	 * @param string $message Error message.
	 * @return void
	 * @since 1.0.0
	 */
	public static function error( $message ) {
		self::set(
			array(
				'type'    => 'error',
				'message' => $message,
			)
		);
	}

	/**
	 * Set done log.
	 *
	 * @param string $message done message.
	 * @return void
	 * @since 1.0.0
	 */
	public static function done( $message ) {
		self::set(
			array(
				'type'    => 'done',
				'message' => $message,
			)
		);
	}

	/**
	 * Reset log. This method should be called at the very end of the process.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public static function reset() {

		if ( isset( $_SESSION['ebwp_logs'] ) ) {
			unset( $_SESSION['ebwp_logs'] );
		}

	}

	/**
	 * Close the logging and reset vars.
	 *
	 * @return void
	 */
	public static function reset_and_close() {
		self::reset();
		self::$is_closed = true;
	}
}
