<?php
/**
 * Class for displaying server related information.
 *
 * @package everest-backup
 */

namespace Everest_Backup;

use Everest_Backup\Traits\Singleton;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for displaying server related information.
 *
 * @since 1.0.0
 */
class Server_Information {


	use Singleton;

	/**
	 * Information data.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $infos = array();

	/**
	 * Init class.
	 *
	 * @since 1.0.0
	 */
	private function __construct() {
		$this->set_domain_info();
		$this->set_server_info();
		$this->set_wp_info();
		$this->set_ebwp_info();
	}

	/**
	 * Information about domain.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function set_domain_info() {
		$this->infos['is_ssl'] = array(
			'label' => __( 'SSL', 'everest-backup' ),
			'value' => is_ssl() ? __( 'Yes', 'everest-backup' ) : __( 'No', 'everest-backup' ),
		);

		$this->infos['home_url'] = array(
			'label' => __( 'Home Url', 'everest-backup' ),
			'value' => home_url( '/' ),
		);

		$this->infos['site_url'] = array(
			'label' => __( 'Site Url', 'everest-backup' ),
			'value' => site_url( '/' ),
		);
	}

	/**
	 * Information about server and host.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function set_server_info() {
		$this->infos['is_writable'] = array(
			'label' => __( 'Writable', 'everest-backup' ),
			'value' => wp_is_writable( '/' ) ? __( 'Yes', 'everest-backup' ) : __( 'No', 'everest-backup' ),
		);

		$this->infos['php_version'] = array(
			'label' => __( 'PHP Version', 'everest-backup' ),
			'value' => PHP_VERSION,
		);

		$this->infos['server_engine'] = array(
			'label' => __( 'Server Engine', 'everest-backup' ),
			'value' => ! empty( $_SERVER['SERVER_SOFTWARE'] ) ? sanitize_text_field( wp_unslash( $_SERVER['SERVER_SOFTWARE'] ) ) : 'N/A',
		);

		$this->infos['total_space'] = array(
			'label' => __( 'Total Space', 'everest-backup' ),
			'value' => size_format( disk_total_space( '/' ) ),
		);

		$this->infos['available_space'] = array(
			'label' => __( 'Available Space', 'everest-backup' ),
			'value' => size_format( disk_free_space( '/' ) ),
		);

		$this->infos['memory_limit'] = array(
			'label' => __( 'Memory Limit', 'everest-backup' ),
			'value' => ini_get( 'memory_limit' ),
		);

		$this->infos['max_upload_size'] = array(
			'label' => __( 'Maximum Upload Size', 'everest-backup' ),
			'value' => size_format( wp_max_upload_size() ),
		);

		$this->infos['operating_system'] = array(
			'label' => __( 'Operating System', 'everest-backup' ),
			'value' => PHP_OS,
		);
	}

	/**
	 * Information about WordPress
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function set_wp_info() {
		global $wp_version, $wp_db_version;

		$this->infos['wp_version'] = array(
			'label' => __( 'WordPress Version', 'everest-backup' ),
			'value' => $wp_version,
		);

		$this->infos['db_version'] = array(
			'label' => __( 'Database Version', 'everest-backup' ),
			'value' => $wp_db_version,
		);

		$this->infos['is_multisite'] = array(
			'label' => __( 'Multisite', 'everest-backup' ),
			'value' => is_multisite() ? __( 'Yes', 'everest-backup' ) : __( 'No', 'everest-backup' ),
		);
	}

	/**
	 * Set Everest Backup related information.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function set_ebwp_info() {

		$backup_dir_path = EVEREST_BACKUP_BACKUP_DIR_PATH;
		$storage_path    = Filesystem::init()->get_storage_dir();

		$this->infos['ebwp_backup_dir_path'] = array(
			'label' => __( 'Backup Directory Path', 'everest-backup' ),
			'value' => $backup_dir_path,
		);

		$this->infos['ebwp_backup_dir_writable'] = array(
			'label' => __( 'Backup Directory Writable', 'everest-backup' ),
			'value' => wp_is_writable( $backup_dir_path ) ? __( 'Yes', 'everest-backup' ) : __( 'No', 'everest-backup' ),
		);

		$this->infos['ebwp_storage_dir_path'] = array(
			'label' => __( 'Storage Directory Path', 'everest-backup' ),
			'value' => $storage_path,
		);

		$this->infos['ebwp_storage_dir_writable'] = array(
			'label' => __( 'Storage Directory Writable', 'everest-backup' ),
			'value' => wp_is_writable( $storage_path ) ? __( 'Yes', 'everest-backup' ) : __( 'No', 'everest-backup' ),
		);

		$this->infos['ebwp_version'] = array(
			'label' => __( 'EBWP Version', 'everest-backup' ),
			'value' => EVEREST_BACKUP_VERSION,
		);

		$this->infos['ebwp_archiver'] = array(
			'label' => __( 'EBWP Archiver', 'everest-backup' ),
			'value' => class_exists( 'ZipArchive' ) ? 'ZipArchive' : 'PclZip', // Class names.
		);

		$this->infos['ebwp_addons'] = array(
			'label' => __( 'EBWP Addons', 'everest-backup' ),
			'value' => everest_backup_installed_addons(),
		);

		$this->infos['ebwp_active_addons'] = array(
			'label' => __( 'EBWP Active Addons', 'everest-backup' ),
			'value' => everest_backup_installed_addons( 'active' ),
		);

		$this->infos['ebwp_paused_addons'] = array(
			'label' => __( 'EBWP Paused Addons', 'everest-backup' ),
			'value' => everest_backup_installed_addons( 'paused' ),
		);
	}

	/**
	 * Returns information array.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	public function get_infos() {
		return $this->infos;
	}

	/**
	 * Print information HTML table.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function display() {
		$infos = $this->get_infos();

		?>
		<div class="everest-backup-information-container">
			<table class="widefat striped" role="presentation">
				<tbody>

					<?php
					if ( is_array( $infos ) && ! empty( $infos ) ) {
						foreach ( $infos as $key => $info ) {
							?>
							<tr id="<?php echo esc_attr( $key ); ?>">
								<td><?php echo wp_kses_post( $info['label'] ); ?></td>

								<?php
								if ( is_array( $info['value'] ) ) {
									$icon = '&#8853;';
									?>
									<td>
										<p><span><?php echo esc_html( $icon ); ?></span> <?php echo wp_kses_post( implode( "</p><p><span>{$icon}</span> ", $info['value'] ) ); ?></p>
									</td>
									<?php
								} else {
									?>
									<td><?php echo wp_kses_post( $info['value'] ); ?></td>
									<?php
								}
								?>
							</tr>
							<?php
						}
					}
					?>

				</tbody>
			</table>
		</div>
		<?php
	}
}
