<?php
/**
 * Functions and definitions for Everest Backup plugin.
 *
 * @package everest-backup
 */

use Everest_Backup\Backup_Directory;
use Everest_Backup\Logs;
use Everest_Backup\Transient;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Returns list of everest-backup installed addons according to the filter.
 *
 * @param string $filter Filter addon. Supports `all`, `active`, and `paused`.
 * @return array
 * @since 1.0.0
 */
function everest_backup_installed_addons( $filter = 'all' ) {

	if ( ! function_exists( 'get_plugins' ) ) {
		require_once ABSPATH . 'wp-admin/includes/plugin.php';
	}

	static $ebwp_addons = array();

	$ebwp_active = array();
	$ebwp_paused = array();

	$prefix = 'everest-backup-';

	if ( ! $ebwp_addons ) {
		$plugins = array_keys( get_plugins() );

		if ( is_array( $plugins ) && ! empty( $plugins ) ) {
			foreach ( $plugins as $plugin ) {
				if ( false !== strpos( $plugin, $prefix ) ) {
					$ebwp_addons[] = $plugin;
				}
			}
		}
	}

	if ( 'all' === $filter ) {
		return $ebwp_addons;
	}

	if ( is_array( $ebwp_addons ) && ! empty( $ebwp_addons ) ) {
		foreach ( $ebwp_addons as $ebwp_addon ) {
			if ( is_plugin_active( $ebwp_addon ) ) {
				$ebwp_active[] = $ebwp_addon;
			} else {
				$ebwp_paused[] = $ebwp_addon;
			}
		}
	}

	return 'paused' === $filter ? $ebwp_paused : $ebwp_active;

}

/**
 * Returns an array of addons data.
 *
 * @param string $category Addons category.
 * @return array
 * @since 1.0.1
 */
function everest_backup_fetch_addons( $category = '' ) {

	/**
	 * Cache function internally.
	 */
	static $json;

	if ( ! $json ) {

		if ( ! class_exists( 'Everest_Backup\Transient' ) ) {
			require_once EVEREST_BACKUP_PATH . 'inc/classes/class-transient.php';
		}

		$transient = new Transient( 'fetch_addons' );

		if ( everest_backup_is_debug_on() ) {
			$transient->delete();
		}

		$json = $transient->get();

		if ( ! $json ) {
			$res = wp_remote_get(
				EVEREST_BACKUP_ADDONS_JSON_URL,
				array(
					'sslverify' => false,
				)
			);

			$json = wp_remote_retrieve_body( $res );

			if ( ! is_array( json_decode( $json, true ) ) ) {
				return;
			}

			$transient->set( $json, HOUR_IN_SECONDS );
		}
	}

	if ( ! $json ) {
		return;
	}

	$decoded = json_decode( $json, true );

	if ( ! is_array( $decoded ) ) {
		return;
	}

	/**
	 * Hook to filter fetched addons data,
	 *
	 * @since 1.0.2
	 */
	$addons = apply_filters( 'everest_backup_filter_fetch_addons', $decoded );

	$data[ $category ] = ! empty( $addons[ $category ] ) ? $addons[ $category ] : '';

	return array(
		'categories' => array_keys( $addons ),
		'data'       => $category ? $data : $addons,
	);

}

/**
 * Returns the addon information according to its provided category and slug.
 *
 * @param string $category Addon category.
 * @param string $slug     Addon slug.
 * @return array
 * @since 1.0.1
 */
function everest_backup_addon_info( $category, $slug ) {
	$all_addons = everest_backup_fetch_addons( $category );

	$addons = ! empty( $all_addons['data'][ $category ] ) ? $all_addons['data'][ $category ] : '';

	if ( ! $addons ) {
		return;
	}

	$info = ! empty( $addons[ $slug ] ) ? $addons[ $slug ] : '';

	if ( ! is_array( $info ) ) {
		return;
	}

	$ebwp_addons = everest_backup_installed_addons();
	$plugin      = $slug . '/' . $slug . '.php';

	$installed = in_array( $plugin, $ebwp_addons, true );

	$info['plugin']    = $plugin;
	$info['installed'] = $installed;
	$info['active']    = $installed && is_plugin_active( $plugin );

	/**
	 * Hook to filter addon information.
	 *
	 * @param array $info Addon information.
	 * @since 1.0.2
	 */
	$info = apply_filters( 'everest_backup_filter_addon_info', $info, compact( 'category', 'slug' ) );

	return $info;

}

/**
 * Fetch contents for sidebar from sidebar.json
 *
 * @param string $page Current everest backup admin page.
 * @return array
 */
function everest_backup_fetch_sidebar( $page ) {

	/**
	 * Cache function internally.
	 */
	static $json;

	if ( ! $json ) {

		if ( ! class_exists( 'Everest_Backup\Transient' ) ) {
			require_once EVEREST_BACKUP_PATH . 'inc/classes/class-transient.php';
		}

		$transient = new Transient( 'fetch_sidebar' );

		if ( everest_backup_is_debug_on() ) {
			$transient->delete();
		}

		$json = $transient->get();

		if ( ! $json ) {
			$res = wp_remote_get(
				EVEREST_BACKUP_SIDEBAR_JSON_URL,
				array(
					'sslverify' => false,
				)
			);

			$json = wp_remote_retrieve_body( $res );

			if ( ! is_array( json_decode( $json, true ) ) ) {
				return;
			}

			$transient->set( $json, HOUR_IN_SECONDS );
		}
	}

	if ( ! $json ) {
		return;
	}

	$decoded = json_decode( $json, true );

	if ( ! is_array( $decoded ) ) {
		return;
	}

	$data = array(
		'global' => ! empty( $decoded['global'] ) ? $decoded['global'] : array(),
		'paged'  => ! empty( $decoded['paged'][ $page ] ) ? $decoded['paged'][ $page ] : array(),
	);

	return $data;

}

/**
 * Checks if debugging is defined.
 *
 * @return bool
 */
function everest_backup_is_debug_on() {
	return defined( 'EVEREST_BACKUP_DEBUG' ) && EVEREST_BACKUP_DEBUG;
}

/**
 * If current connection is localhost.
 *
 * @return bool
 * @since 1.0.0
 */
function everest_backup_is_localhost() {
	$whitelist   = array( '127.0.0.1', '::1' );
	$remote_addr = ! empty( $_SERVER['REMOTE_ADDR'] ) ? sanitize_text_field( wp_unslash( $_SERVER['REMOTE_ADDR'] ) ) : '';
	if ( in_array( $remote_addr, $whitelist, true ) ) {
		return true;
	}
}

/**
 * Add query args to API auth redirect url.
 *
 * @uses add_query_arg
 * @param array $args An associative array of key/value pairs: `array( 'key1' => 'value1', 'key2' => 'value2', )`.
 * @return string Returns new url.
 * @since 1.0.0
 */
function everest_backup_add_redirect_url_query_arg( $args = array() ) {
	$redirect_url = EVEREST_BACKUP_AUTH_REDIRECT_URL;

	return add_query_arg( $args, $redirect_url );
}

/**
 * Removes url argument passed as $key from the url.
 *
 * @param string|string[] $key — Query key or keys to remove.
 * @return string  New URL query string.
 * @since
 */
function everest_backup_remove_redirect_url_query_arg( $key ) {
	$redirect_url = EVEREST_BACKUP_AUTH_REDIRECT_URL;

	return remove_query_arg( $key, $redirect_url );
}

/**
 * Recursively sanitize the array values.
 *
 * @param array $data Array data to sanitize.
 * @return array $data Sanitized array data.
 * @since 1.0.0
 */
function everest_backup_sanitize_array( array $data ) {
	foreach ( $data as $key => &$value ) {
		if ( is_array( $value ) ) {
			$value = everest_backup_sanitize_array( $value );
		} else {
			if ( is_int( $value ) ) {
				$value = (int) $value;
			} elseif ( is_string( $value ) ) {
				$value = sanitize_text_field( wp_unslash( $value ) );
			}
		}
	}

	return $data;
}

/**
 * Returns settings data.
 *
 * @param string $key Array key for the settings data.
 * @return mixed If key is provided or not empty then it returns all the data according to key else it will return all settings data.
 * @since 1.0.0
 */
function everest_backup_get_settings( $key = '' ) {
	$settings = get_option( EVEREST_BACKUP_SETTINGS_KEY, array() );

	if ( $key ) {
		return isset( $settings[ $key ] ) ? $settings[ $key ] : '';
	}

	return $settings;
}

/**
 * Update settings.
 *
 * @param array $settings Settings data to save.
 * @return void
 * @since 1.0.0
 */
function everest_backup_update_settings( $settings ) {

	if ( ! $settings ) {
		return;
	}

	$sanitized_settings = everest_backup_sanitize_array( $settings );

	update_option( EVEREST_BACKUP_SETTINGS_KEY, $sanitized_settings );
}

/**
 * Returns maximum file upload size.
 *
 * @return int If returns 0 then it means "unlimited" or no limit.
 * @since 1.0.0
 */
function everest_backup_max_upload_size() {

	$wp_limit = wp_max_upload_size();

	return (int) apply_filters( 'everest_backup_filter_max_upload_size', $wp_limit );
}

/**
 * Returns cron cycles for schedule backup.
 *
 * @return array
 * @since 1.0.0
 */
function everest_backup_cron_cycles() {
	return apply_filters(
		'everest_backup_filter_cron_cycles',
		array(
			'everest_backup_daily'   => array(
				'interval' => DAY_IN_SECONDS, // 24 hours.
				'display'  => __( 'Daily', 'everest-backup' ),
			),
			'everest_backup_weekly'  => array(
				'interval' => WEEK_IN_SECONDS, // 1 week.
				'display'  => __( 'Weekly', 'everest-backup' ),
			),
			'everest_backup_monthly' => array(
				'interval' => MONTH_IN_SECONDS, // 1 month.
				'display'  => __( 'Monthly', 'everest-backup' ),
			),
		)
	);
}

/**
 * Returns an array of locations to save the backup packages.
 *
 * @return array
 * @since 1.0.0
 */
function everest_backup_package_locations() {
	return (array) apply_filters(
		'everest_backup_filter_package_locations',
		array(
			'server' => array(
				'label'       => __( 'Local Web Server', 'everest-backup' ),
				'description' => __( 'Save the backup package locally on your host server.', 'everest-backup' ),
				'is_active'   => true,
			),
		)
	);
}

/**
 * Where should the current package be saved.
 * If current operation is Ajax then it returns `Additional Settings` value,
 * If current operation is Cron then it returns `Settings > Schedule Backup` value.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_is_saving_to() {
	if ( wp_doing_ajax() ) {
		$response = everest_backup_get_ajax_response(
			array(
				'everest_backup_backup_init',
				'everest_backup_backup_final',
			)
		);
		return ! empty( $response['save_to'] ) ? $response['save_to'] : 'server';
	}

	if ( wp_doing_cron() ) {
		$schedule_backup_data = everest_backup_get_settings( 'schedule_backup' );
		return ! empty( $schedule_backup_data['save_to'] ) ? $schedule_backup_data['save_to'] : 'server';
	}
}

/**
 * Array search. Compatible with multi dimensional arrays.
 *
 * @param array      $array Array data to run the search.
 * @param string|int $field Field to check as key in the array.
 * @param mixed      $values Value respective to the field.
 * @return int|string $key Array key.
 * @since 1.0.0
 */
function everest_backup_array_search( $array, $field, $values ) {
	if ( is_array( $array ) && ! empty( $array ) ) {
		foreach ( $array as $key => $val ) {
			if ( ! isset( $val[ $field ] ) ) {
				continue;
			}

			if ( is_array( $values ) ) {
				if ( is_array( $values ) && ! empty( $values ) ) {
					foreach ( $values as $value ) {
						if ( $value === $val[ $field ] ) {
							return $key;
						}
					}
				}
			}

			if ( $values === $val[ $field ] ) {
				return $key;
			}
		}
	}
}

/**
 * List of items to exlude from backup. Eg; Plugins, Themes...
 *
 * @return array
 * @since 1.0.0
 */
function everest_backup_get_backup_excludes() {
	return apply_filters(
		'everest_backup_filter_backup_excludes',
		array(
			'ignore_plugins'  => array(
				'label'       => __( 'Plugins (Files)', 'everest-backup' ),
				'description' => __( 'Ignore plugins', 'everest-backup' ),
			),
			'ignore_themes'   => array(
				'label'       => __( 'Themes (Files)', 'everest-backup' ),
				'description' => __( 'Ignore themes', 'everest-backup' ),
			),
			'ignore_media'    => array(
				'label'       => __( 'Media (Files)', 'everest-backup' ),
				'description' => __( 'Ignore media', 'everest-backup' ),
			),
			'ignore_database' => array(
				'label'       => __( 'Database (Sql)', 'everest-backup' ),
				'description' => __( 'Ignore database', 'everest-backup' ),
			),
		)
	);
}

/**
 * Process types used by logs.
 *
 * @return array
 * @since 1.0.0
 */
function everest_backup_get_process_types() {
	return apply_filters(
		'everest_backup_filter_process_types',
		array(
			'abort'           => __( 'Abort', 'everest-backup' ),
			'backup'          => __( 'Backup', 'everest-backup' ),
			'rollback'        => __( 'Rollback', 'everest-backup' ),
			'restore'         => __( 'Restore', 'everest-backup' ),
			'clone'           => __( 'Clone', 'everest-backup' ),
			'schedule_backup' => __( 'Schedule Backup', 'everest-backup' ),
		)
	);
}

/**
 * Returns true if doing the rollback.
 *
 * @return bool
 * @since 1.0.0
 */
function everest_backup_doing_rollback() {
	return defined( 'EVEREST_BACKUP_DOING_ROLLBACK' ) && EVEREST_BACKUP_DOING_ROLLBACK;
}

/**
 * Returns array of info of provided backup file path.
 *
 * @param string $backup_file Full path to the backup file.
 * @return array
 * @since 1.0.0
 */
function everest_backup_get_backup_file_info( $backup_file ) {

	if ( ! is_file( $backup_file ) ) {
		return array();
	}

	return array(
		'filename' => basename( $backup_file ),
		'path'     => $backup_file,
		'url'      => everest_backup_convert_file_path_to_url( $backup_file ),
		'size'     => filesize( $backup_file ),
		'time'     => filemtime( $backup_file ),
	);
}

/**
 * Check if file extension is excluded in settings by the user.
 *
 * @param string $file Full path to the backup file.
 * @return bool
 * @since 1.0.0
 */
function everest_backup_is_extension_excluded( $file ) {
	if ( ! is_file( $file ) ) {
		return false;
	}

	$general    = everest_backup_get_settings( 'general' );
	$extensions = ! empty( $general['exclude_files_by_extension'] ) ? $general['exclude_files_by_extension'] : '';

	if ( ! $extensions ) {
		return false;
	}

	$excluded = explode( ', ', $extensions );

	$extension = pathinfo( $file, PATHINFO_EXTENSION );

	return in_array( $extension, $excluded, true );
}

/**
 * Returns backup file full path according to the passed backup file name.
 *
 * @param string $backup_filename File name of backup package.
 * @param bool   $check If passed true then it checks if file exist or not.
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_backup_full_path( $backup_filename, $check = true ) {

	$backup_dir = EVEREST_BACKUP_BACKUP_DIR_PATH;

	$backup_file_path = wp_normalize_path( "{$backup_dir}/{$backup_filename}" );

	if ( ! $check ) {
		return $backup_file_path;
	}

	return is_file( $backup_file_path ) ? $backup_file_path : '';

}

/**
 * Sends json success response with logs.
 *
 * @param mixed $data Optional. Data to encode as JSON, then print and die. Default null.
 * @param int   $status_code Optional. The HTTP status code to output. Default null.
 * @param int   $options Optional. Options to be passed to json_encode(). Default 0.
 * @return void
 * @since 1.0.0
 */
function everest_backup_send_success( $data = null, $status_code = null, $options = 0 ) {

	$res  = array();
	$logs = Logs::get();

	$res['logs']   = $logs;
	$res['result'] = $data;

	Logs::save_to_log_json();
	Logs::save();
	Logs::reset_and_close();

	if ( everest_backup_doing_rollback() ) {

		/**
		 * If we are doing rollback then lets redirect to logs page.
		 */
		if ( wp_safe_redirect( network_admin_url( '/admin.php?page=everest-backup-logs' ) ) ) {
			exit;
		}
	}

	/**
	 * As it is possible that "everest_backup_send_success" and "everest_backup_send_error"
	 * could both be trigger in same script run, which can cause mis-matched json error in client slide.
	 * So to avoid that, we are not sending any response from this function if the $logs is empty.
	 */
	if ( ! $logs ) {
		die;
	}

	wp_send_json_success( $res, $status_code, $options );
}

/**
 * Sends json error response with logs.
 *
 * @param mixed $data Optional. Data to encode as JSON, then print and die. Default null.
 * @param int   $status_code Optional. The HTTP status code to output. Default null.
 * @param int   $options Optional. Options to be passed to json_encode(). Default 0.
 * @return void
 * @since 1.0.0
 */
function everest_backup_send_error( $data = null, $status_code = null, $options = 0 ) {

	$res  = array();
	$logs = Logs::get();

	$res['logs']   = $logs;
	$res['result'] = $data;

	Logs::save_to_log_json();
	Logs::save();
	Logs::reset_and_close();

	if ( everest_backup_doing_rollback() ) {

		/**
		 * If we are doing rollback then lets redirect to logs page.
		 */
		if ( wp_safe_redirect( network_admin_url( '/admin.php?page=everest-backup-logs' ) ) ) {
			exit;
		}
	}

	/**
	 * As it is possible that "everest_backup_send_success" and "everest_backup_send_error"
	 * could both be trigger in same script run, which can cause mis-matched json error in client slide.
	 * So to avoid that, we are not sending any response from this function if the $logs is empty.
	 */
	if ( ! $logs ) {
		die;
	}

	wp_send_json_error( $res, $status_code, $options );
}

/**
 * Returns post/get/request data.
 *
 * @param string $type Type of submission request to return.
 * @return array
 * @since 1.0.0
 */
function everest_backup_get_submitted_data( $type = 'request' ) {
	$data = array();

	switch ( $type ) {
		case 'post':
			$data = $_POST; // @phpcs:ignore
			break;

		case 'get':
			$data = $_GET; // @phpcs:ignore
			break;

		default:
			$data = $_REQUEST; // @phpcs:ignore
			break;
	}

	return $data;
}

/**
 * Verify ajax nonce and return ajax body response.
 *
 * @param string|array $action Ajax action. If array is provided then the response action will be checked with each array element.
 * @return array body response.
 * @since 1.0.0
 */
function everest_backup_get_ajax_response( $action ) {
	if ( ! wp_doing_ajax() ) {
		return;
	}

	if ( ! everest_backup_verify_nonce( 'everest_backup_ajax_nonce' ) ) {
		$message = __( 'Nonce verification failed.', 'everest-backup' );
		Logs::error( $message );
		everest_backup_send_error( $message );
	}

	/**
	 * If nonce verified, lets bring things into action. No puns intended.
	 */
	$request = everest_backup_get_submitted_data();

	$inputstream = file_get_contents( 'php://input' );
	$data_decode = (array) json_decode( $inputstream, true );
	$response    = array_merge( $request, $data_decode );

	$res_action = ! empty( $response['action'] ) ? sanitize_text_field( wp_unslash( $response['action'] ) ) : '';

	$is_action_valid = is_array( $action ) ? in_array( $res_action, $action, true ) : ( $res_action === $action );

	if ( ! $is_action_valid ) {
		$message = __( 'Invalid action provided.', 'everest-backup' );
		Logs::error( $message );
		everest_backup_send_error( $message );
	}

	return $response;

}

/**
 * Returns admin email set by user or WP admin email as default.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_admin_email() {
	$general = everest_backup_get_settings( 'general' );

	return ! empty( $general['admin_email'] ) ? $general['admin_email'] : get_option( 'admin_email' );
}


/**
 * PHP setup environment
 *
 * @return void
 */
function everest_backup_setup_environment() {

	/**
	 * Do not stop the script even if user exit the tab.
	 */
	ignore_user_abort( true );

	/**
	 * No time limit for script execution.
	 */
	set_time_limit( 0 );

	/**
	 * Set maximum time in seconds a script is allowed to parse input data
	 */
	ini_set( 'max_input_time', '-1' ); // @phpcs:ignore

	/**
	 * Clean (erase) the output buffer and turn off output buffering
	 */
	if ( ob_get_length() ) {
		ob_end_clean();
	}

	/**
	 * Custom error handler.
	 */
	set_error_handler( // @phpcs:ignore
		function( $errno, $message, $file, $line ) {
			if ( ! $message ) {
				return;
			}

			/* translators: %1$s is the error message, %2$s is the file path and %3$s is the file line number. */
			$error = sprintf( __( '%1$s in %2$s on line %3$s' ), esc_html( $message ), esc_url_raw( $file ), absint( $line ) );

			switch ( $errno ) {

				case E_USER_WARNING:
					Logs::warn( $error );
					break;

				case E_USER_NOTICE:
					Logs::warn( $error );
					break;

				case E_USER_ERROR:
					Logs::error( $error );
					break;

				default:
					Logs::error( $error );
					break;
			}

			return true;

		}
	);

	/**
	 * Catch errors and save them.
	 */
	register_shutdown_function(
		function() {
			$last_error = error_get_last();

			if ( ! is_array( $last_error ) ) {
				return;
			}

			/* translators: %1$s is the error message, %2$s is the file path and %3$s is the file line number. */
			$error = sprintf( __( '%1$s in %2$s on line %3$s' ), esc_html( $last_error['message'] ), esc_url_raw( $last_error['file'] ), absint( $last_error['line'] ) );

			Logs::error( $error );

			if ( E_ERROR === $last_error['type'] ) {

				/**
				 * Send response during fatal errors only.
				 */
				everest_backup_send_error();
			}
		}
	);
}

/**
 * Check if connection or process has been aborted by the user.
 *
 * @return bool
 * @since 1.0.0
 *
 * @see https://stackoverflow.com/a/16592945
 */
function everest_backup_has_aborted() {

	if ( wp_doing_cron() ) {
		return;
	}

	print ' ';
	flush();
	if ( ob_get_level() > 0 ) {
		ob_flush();
	}

	return connection_aborted() === 1;

}

/**
 * Set current archive process file name with full path.
 *
 * @param string $zipname Archive zip filename with full path.
 * @return void
 * @since 1.0.0
 */
function everest_backup_set_current_archive( $zipname ) {

	if ( wp_doing_cron() ) {
		return;
	}

	if ( ! session_id() ) {
		session_start();
	}

	$_SESSION['ebwp_current_archive'] = $zipname;

}

/**
 * Get current archive process file name with full path.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_current_archive() {
	return isset( $_SESSION['ebwp_current_archive'] ) ? esc_url_raw( $_SESSION['ebwp_current_archive'] ) : '';
}

/**
 * Wrapper function for `wp_nonce_field`
 *
 * @param string $action Scalar value to add context to the nonce.
 * @param bool   $referer Optional. Whether to set the referer field for validation. Default true.
 * @return string The token.
 * @since 1.0.0
 */
function everest_backup_nonce_field( $action, $referer = true ) {
	if ( $action ) {
		return wp_nonce_field( $action, $action, $referer );
	}
}

/**
 * Wrapper function for `wp_create_nonce`
 *
 * @param string $action Scalar value to add context to the nonce.
 * @return string The token.
 * @since 1.0.0
 */
function everest_backup_create_nonce( $action ) {
	if ( $action ) {
		return wp_create_nonce( $action );
	}
}

/**
 * Verify nonce.
 *
 * @param string $action Should give context to what is taking place and be the same when nonce was created.
 *
 * @return bool
 * @since 1.0.0
 */
function everest_backup_verify_nonce( $action ) {

	$nonce = ! empty( $_REQUEST[ $action ] ) ? sanitize_text_field( wp_unslash( $_REQUEST[ $action ] ) ) : '';

	if ( $nonce && $action ) {
		return wp_verify_nonce( $nonce, $action );
	}
}

/**
 * Returns WordPress uploads directory path.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_uploads_dir() {
	$upload_dir = wp_upload_dir( null, false );
	if ( $upload_dir ) {
		if ( ! empty( $upload_dir['basedir'] ) ) {
			return trailingslashit( $upload_dir['basedir'] );
		}
	}
}

/**
 * Returns WordPress uploads directory url.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_uploads_url() {
	$upload_dir = wp_upload_dir( null, false );
	if ( $upload_dir ) {
		if ( ! empty( $upload_dir['baseurl'] ) ) {
			return trailingslashit( $upload_dir['baseurl'] );
		}
	}
}

/**
 * Converts file paths to url.
 *
 * @param string $file Full path to file.
 * @return string
 * @since 1.0.0
 */
function everest_backup_convert_file_path_to_url( $file ) {

	if ( ! $file ) {
		return $file;
	}

	$abspath = wp_normalize_path( trailingslashit( ABSPATH ) );
	$homeurl = wp_normalize_path( trailingslashit( home_url() ) );

	return str_replace( $abspath, $homeurl, $file );
}

/**
 * Returns contents of WordPress .htaccess file.
 *
 * @return string
 * @since 1.0.0
 */
function everest_backup_get_htaccess() {
	$htaccess = EVEREST_BACKUP_HTACCESS_PATH;

	if ( is_file( $htaccess ) ) {
		return file_get_contents( $htaccess ); // @phpcs:ignore
	}
}

/**
 * Converts strings to hex string.
 *
 * @param string $string String to convert into hex.
 * @return string
 * @since 1.0.0
 */
function everest_backup_str2hex( $string ) {
	$hexstr = unpack( 'H*', $string );
	return array_shift( $hexstr );
}

/**
 * Converts hex string to plain string.
 *
 * @param string $string Hex string to convert into plain string.
 * @return string
 * @since 1.0.0
 */
function everest_backup_hex2str( $string ) {
	return hex2bin( "$string" );
}

/**
 * Locate and render files from view folder.
 *
 * @param string $template Template file name with extension.
 * @param array  $args Arguments that will be passed to the template.
 * @return void
 * @since 1.0.0
 */
function everest_backup_render_view( $template, $args = array() ) {
	$file = wp_normalize_path( EVEREST_BACKUP_VIEWS_DIR . $template . '.php' );

	if ( ! file_exists( $file ) ) {
		return;
	}

	$args = apply_filters( 'everest_backup_filter_view_renderer_args', $args, $template );

	everest_backup_print_notice();

	do_action( 'everest_backup_before_view_rendered', $template, $args );

	load_template( $file, false, $args );

	do_action( 'everest_backup_after_view_rendered', $template, $args );

}

/**
 * Set one time, dismissable notice.
 *
 * @param string $notice Notice message.
 * @param string $type Notice type.
 *
 * @see https://developer.wordpress.org/reference/hooks/admin_notices/#example for all notice types.
 *
 * @return void
 * @since 1.0.0
 */
function everest_backup_set_notice( $notice, $type ) { // @phpcs:ignore
	if ( ! session_id() ) {
		session_start();
	}

	$notices                 = isset( $_SESSION['ebwp_notice'] ) ? everest_backup_sanitize_array( $_SESSION['ebwp_notice'] ) : array();
	$_SESSION['ebwp_notice'] = compact( 'notice', 'type' );

}

/**
 * Prints HTML for the package location( Save To ) dropdown.
 *
 * @param array $args Arguments for the dropdown.
 *
 * @type `array $args['package_locations']` [Optional] Package locations array.
 * @type `string $args['name']` HTML name sttribute.
 * @type `string $args['id']` HTML id sttribute.
 * @type `string $args['class']` HTML class sttribute.
 * @type `string $args['selected']`Currently selected item.
 *
 * @return void
 * @since 1.0.0
 */
function everest_backup_package_location_dropdown( $args ) {

	$parsed_args = wp_parse_args(
		$args,
		array(
			'package_locations' => array(),
			'name'              => '',
			'id'                => '',
			'class'             => '',
			'selected'          => '',
		)
	);

	$package_locations = ! empty( $parsed_args['package_locations'] ) ? $parsed_args['package_locations'] : everest_backup_package_locations();
	$name              = $parsed_args['name'];
	$id                = $parsed_args['id'];
	$class             = $parsed_args['class'];
	$selected          = $parsed_args['selected'];

	ob_start();
	?>
	<select name="<?php echo esc_attr( $name ); ?>" id="<?php echo esc_attr( $id ); ?>" class="<?php echo esc_attr( $class ); ?>">
		<?php
		if ( is_array( $package_locations ) && ! empty( $package_locations ) ) {
			foreach ( $package_locations as $key => $package_location ) {
				?>
				<option
					<?php
					selected( $selected, $key );
					disabled( ( false === $package_location['is_active'] ) );
					?>
					value="<?php echo esc_attr( $key ); ?>"
					title="<?php echo esc_attr( $package_location['description'] ); ?>"
				>
					<?php echo esc_html( $package_location['label'] ); ?> (&#8505;)
				</option>
				<?php
			}
		}
		?>
	</select>
	<?php
	$content = ob_get_clean();

	echo wp_kses(
		$content,
		array(
			'select' => array(
				'name'  => array(),
				'id'    => array(),
				'class' => array(),
			),
			'option' => array(
				'selected' => array(),
				'disabled' => array(),
				'value'    => array(),
				'title'    => array(),
			),
		)
	);
}

/**
 * Prints HTML for the backup files dropdown.
 *
 * @param array $args Arguments for the dropdown.
 *
 * @type `string $args['name']` HTML name sttribute.
 * @type `string $args['id']` HTML id sttribute.
 * @type `string $args['class']` HTML class sttribute.
 * @type `string $args['required']` Is dropdown required or not.
 * @type `string $args['selected']` Currently selected item.
 *
 * @return void
 * @since 1.0.0
 */
function everest_backup_backup_files_dropdown( $args ) {

	$parsed_args = wp_parse_args(
		$args,
		array(
			'name'     => '',
			'id'       => '',
			'class'    => '',
			'required' => false,
			'selected' => '',
			'order'    => 'asc',
		)
	);

	$name         = $parsed_args['name'];
	$id           = $parsed_args['id'];
	$class        = $parsed_args['class'];
	$required     = $parsed_args['required'];
	$selected     = $parsed_args['selected'];
	$order        = $parsed_args['order'];
	$backup_files = Backup_Directory::init()->get_backups_by_order( $order );

	$grouped_backups = array();

	if ( is_array( $backup_files ) && ! empty( $backup_files ) ) {
		foreach ( $backup_files as $backup_file ) {
			$file_info  = everest_backup_get_backup_file_info( $backup_file );
			$group_date = wp_date( 'F j, Y', $file_info['time'] );

			$grouped_backups[ $group_date ][] = $file_info;
		}
	}

	ob_start();
	?>
	<select name="<?php echo esc_attr( $name ); ?>" id="<?php echo esc_attr( $id ); ?>" class="<?php echo esc_attr( $class ); ?>" required="<?php echo $required ? esc_attr( 'required' ) : ''; ?>" >
		<option value=""><?php esc_html_e( '--- Select ---', 'everest-backup' ); ?></option>
		<?php
		if ( is_array( $grouped_backups ) && ! empty( $grouped_backups ) ) {
			foreach ( $grouped_backups as $grouped_backup_date => $grouped_backup ) {
				?>
				<optgroup label="<?php echo esc_attr( $grouped_backup_date ); ?>">
				<?php
				if ( is_array( $grouped_backup ) && ! empty( $grouped_backup ) ) {
					foreach ( $grouped_backup as $backup ) {

						$filename  = $backup['filename'];
						$opt_label = $filename . ' [ ' . wp_date( 'h:i:s A', $backup['time'] ) . ' ]';
						?>
						<option title="<?php echo esc_attr( size_format( $backup['size'] ) ); ?>" <?php selected( $selected, $filename ); ?> value="<?php echo esc_attr( $filename ); ?>"><?php echo esc_html( $opt_label ); ?></option>
						<?php
					}
				}
				?>
				</optgroup>
				<?php
			}
		}
		?>
	</select>
	<?php
	$content = ob_get_clean();

	echo wp_kses(
		$content,
		array(
			'select'   => array(
				'name'     => array(),
				'id'       => array(),
				'class'    => array(),
				'required' => array(),
			),
			'optgroup' => array(
				'label' => array(),
			),
			'option'   => array(
				'selected' => array(),
				'disabled' => array(),
				'value'    => array(),
				'title'    => array(),
			),
		)
	);

}

/**
 * Breadcrumb function for Everest Backup admin pages.
 *
 * @return void
 * @since 1.0.0
 */
function everest_backup_breadcrumb() {
	$get = everest_backup_get_submitted_data( 'get' );

	$root = 'Everest Backup';
	$page = get_admin_page_title();
	$tab  = ! empty( $get['tab'] ) ? str_replace( array( '_', '-' ), ' ', ucwords( $get['tab'], '_' ) ) : '';

	?>
	<div class="everest-backup-breadcrumb" id="eb-breadcrumb">
		<svg xmlns="http://www.w3.org/2000/svg" width="17" viewBox="0 0 52.415 40.759">
		<path id="eb_icon_home" d="M25.511,12.828,8.735,26.645V41.557a1.456,1.456,0,0,0,1.456,1.456l10.2-.026a1.456,1.456,0,0,0,1.449-1.456V32.822a1.456,1.456,0,0,1,1.456-1.456h5.823a1.456,1.456,0,0,1,1.456,1.456v8.7a1.456,1.456,0,0,0,1.456,1.46l10.193.028a1.456,1.456,0,0,0,1.456-1.456V26.635L26.9,12.828A1.109,1.109,0,0,0,25.511,12.828Zm26.5,9.391L44.4,15.949V3.345a1.092,1.092,0,0,0-1.092-1.092h-5.1a1.092,1.092,0,0,0-1.092,1.092V9.952l-8.146-6.7a4.368,4.368,0,0,0-5.55,0L.4,22.219A1.092,1.092,0,0,0,.25,23.757l2.32,2.821a1.092,1.092,0,0,0,1.539.148L25.511,9.1a1.109,1.109,0,0,1,1.392,0l21.4,17.629a1.092,1.092,0,0,0,1.538-.146l2.32-2.821a1.092,1.092,0,0,0-.155-1.54Z" transform="translate(0.001 -2.254)" fill="#000000"/>
		</svg>

		<strong><?php echo esc_html( $root ); ?></strong> <span class="breadcrumb-separator">&#187;</span> <?php echo $tab ? wp_kses_post( "<strong>$page</strong> <span class='breadcrumb-separator'>&#187;</span> <small>$tab</small>" ) : wp_kses_post( "<small>$page</small>" ); ?>
	</div>
	<?php

}

/**
 * Prints HTML for toggle switch.
 *
 * @param array $args Arguments for the toggle switch.
 * @return void
 * @since 1.0.0
 */
function everest_backup_switch( $args = array() ) {

	static $called = 0;

	$parsed_args = wp_parse_args(
		$args,
		array(
			'id'              => sprintf( 'ebwp-switch-%d', $called ),
			'class'           => '',
			'name'            => '',
			'value'           => 1,
			'checked'         => false,
			'label_checked'   => __( 'Enable', 'everest-backup' ),
			'label_unchecked' => __( 'Disable', 'everest-backup' ),
		)
	);

	$called++;

	$id    = $parsed_args['id'];
	$class = trim( sprintf( 'toggle %s', $parsed_args['class'] ) );

	ob_start();
	?>
	<div class="toggle-switch">
		<input <?php checked( $parsed_args['checked'] ); ?> name="<?php echo esc_attr( $parsed_args['name'] ); ?>" type="checkbox" class="<?php echo esc_attr( $class ); ?>" id="<?php echo esc_attr( $id ); ?>" value="<?php echo esc_attr( $parsed_args['value'] ); ?>">
		<label for="<?php echo esc_attr( $id ); ?>" data-checked="<?php echo esc_attr( $parsed_args['label_checked'] ); ?>" data-unchecked="<?php echo esc_attr( $parsed_args['label_unchecked'] ); ?>"></label>
	</div>
	<?php
	$content = ob_get_clean();

	echo wp_kses(
		$content,
		array(
			'div'   => array(
				'class' => array(),
			),
			'input' => array(
				'checked' => array(),
				'name'    => array(),
				'type'    => array(),
				'class'   => array(),
				'id'      => array(),
				'value'   => array(),
			),
			'label' => array(
				'for'            => array(),
				'value'          => array(),
				'data-checked'   => array(),
				'data-unchecked' => array(),
			),
		)
	);
}

/**
 * Print one time, dismissable notice.
 *
 * @return void
 * @since
 */
function everest_backup_print_notice() {
	if ( ! isset( $_SESSION['ebwp_notice'] ) ) {
		return;
	}

	$notice = everest_backup_sanitize_array( $_SESSION['ebwp_notice'] );

	?>
	<div class="notice is-dismissible <?php echo esc_attr( $notice['type'] ); ?>">
		<p><?php echo wp_kses_post( $notice['notice'] ); ?></p>
	</div>
	<?php

	unset( $_SESSION['ebwp_notice'] );
}

/**
 * Replace the first occurrence of a string within a string.
 *
 * @param string $search The value being searched for, otherwise known as the needle.
 * @param string $replace The replacement value that replaces found search values.
 * @param string $subject The string or array being searched and replaced on, otherwise known as the haystack.
 * @return string
 * @since 1.0.0
 */
function everest_backup_str_replace_once( $search, $replace, $subject ) {
	$pos = strpos( $subject, $search );
	if ( false !== $pos ) {
		return substr_replace( $subject, $replace, $pos, strlen( $search ) );
	}
}
