<?php
/**
 * Class for handling database import.
 *
 * @package everest-backup
 */

namespace Everest_Backup\Modules;

use Everest_Backup\Database;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for handling database import.
 *
 * @since 1.0.0
 */
class Import_Database extends Database {

	/**
	 * If repaired has completed.
	 *
	 * @var boolean
	 * @since 1.0.2
	 */
	private $repaired = false;

	/**
	 * List of prefixed tables during export.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $exported_tables;

	/**
	 * Find and replace value from the sql string. Array key as the string to find and array value as string to replace with.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	protected $find_replace;

	/**
	 * Init class.
	 *
	 * @param string $filename Path to the sql dump file.
	 * @param array  $exported_tables List of prefixed tables during export.
	 * @param array  $find_replace Find and replace value from the sql string. Array key as the string to find and array value as string to replace with.
	 * @since 1.0.0
	 */
	public function __construct( $filename, $exported_tables, $find_replace = array() ) {
		global $wpdb;

		$this->filename        = $filename;
		$this->exported_tables = $exported_tables;
		$this->find_replace    = $find_replace;

		$this->old_db_prefix = 'EBWP_DB_PREFIX_';
		$this->new_db_prefix = $wpdb->prefix;

		parent::__construct( $wpdb );
	}

	/**
	 * Returns insert sql statement in chunks.
	 *
	 * @param array $insert_arr
	 * @return array
	 */
	protected function get_insert_chunks( $insert_arr ) {

		if ( ! $insert_arr ) {
			return;
		}

		$insert_chunks = array();

		if ( is_array( $insert_arr ) && ! empty( $insert_arr ) ) {
			foreach ( $insert_arr as $insert_arr_k => $insert_arr_v ) {

				$chunks = array_chunk( $insert_arr_v, 5 );

				if ( is_array( $chunks ) && ! empty( $chunks ) ) {
					foreach ( $chunks as $chunk ) {
						$join = implode( " ", $chunk );

						$insert_chunks[] = $insert_arr_k . substr( $join, 0, -1 ) . ';';

						unset( $join );

					}
				}

			}
		}

		return $insert_chunks;

	}

	/**
	 * Custom query function for importing.
	 *
	 * @param string $query
	 * @return mixed
	 */
	public function query( $query ) {

		$dbh = $this->wpdb->dbh;

		if ( ! $dbh ) {
			return;
		}

		if ( $this->wpdb->use_mysqli ) {
			return mysqli_query( $dbh, $query );
		}

		if ( function_exists( 'mysql_query' ) ) {
			return mysql_query( $query, $dbh );
		}

	}

	/**
	 * Import database.
	 *
	 * @return bool $success Returns true on success and false on query error.
	 * @since 1.0.0
	 */
	public function import() {

		$this->query( "SET SESSION sql_mode = ''" );

		$success = false;

		$find      = array_keys( $this->find_replace );
		$replace   = array_values( $this->find_replace );

		$insert     = '';
		$temp_query = '';
		$insert_arr = array();

		$handle = fopen( $this->filename, 'r' );

		if ( $handle ) {
			while ( ! feof( $handle ) ) {
				$sql_line = fgets( $handle );

				if ( ! is_string( $sql_line ) ) {
					continue;
				}

				$sql_line = trim( $sql_line );

				if ( ! $this->is_valid( $sql_line ) ) {
					continue;
				}

				$input = $this->rename_table_names( $sql_line, $this->exported_tables );
				$input = $this->replace_users_login_related_metas( $input );

				if ( false !== strstr( $input, 'INSERT INTO' ) ) {
					$insert = $input;
				}

				if ( $insert ) {
					if ( $input !== $insert ) {
						$insert_arr[ $insert ][] = $input;
					}
				}

				$temp_query .= $input;

				if ( $this->is_query_end( $temp_query ) ) {

					if ( false === strstr( $temp_query, 'INSERT INTO' ) ) {
						$query = $this->query( $temp_query );
					} else {

						$insert_chunks = $this->get_insert_chunks( $insert_arr );

						if ( is_array( $insert_chunks ) && ! empty( $insert_chunks ) ) {
							foreach ( $insert_chunks as $insert_chunk ) {
								$insert_chunk_replace = str_replace( $find, $replace, $insert_chunk );
								$query                = $this->query( $insert_chunk_replace );
							}
						}
					}

					$success = isset( $query ) && is_int( $query );

					$insert     = '';
					$temp_query = '';
					$insert_arr = array();

				}
			}
			fclose( $handle );
		}

		$success = $this->repair();

		return $success;
	}

	/**
	 * Repair database after import.
	 * This method is used especially for repairing serialize data.
	 *
	 * @return void
	 * @since 1.0.2
	 */
	protected function repair() {

		if ( $this->repaired ) {
			/**
			 * Only call this method once.
			 */
			return true;
		}

		$filename = $this->filename;

		if ( is_file( $filename ) ) {
			unlink( $filename );
		}

		/**
		 * During repair, set old_db_prefix same as new_db_prefix before dumping the sql, otherwise it will dump without table names.
		 */
		$this->old_db_prefix = $this->new_db_prefix;

		if ( ! $this->dump_sql() ) {
			$this->repaired = true;
			return true;
		}

		$handle = fopen( $filename, 'r' );

		if ( is_file( $filename ) ) {
			unlink( $filename );
		}

		if ( $handle ) {
			while ( ! feof( $handle ) ) {
				$sql_line = fgets( $handle );

				if ( ! is_string( $sql_line ) ) {
					continue;
				}

				$created = $this->write( $sql_line );

			}
			fclose( $handle );
		}

		if ( isset( $created ) && is_int( $created ) ) {
			$this->repaired = true;
			$this->import();
		}

		$this->repaired = true;
		return true;

	}

}
