<?php
/**
 * Class for handling the clone functionality.
 *
 * @package everest-backup
 */

namespace Everest_Backup\Modules;

use Everest_Backup\Logs;
use Everest_Backup\Migration_Clone;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Class for handling the clone functionality.
 *
 * @since 1.0.0
 */
class Cloner extends Migration_Clone {

	/**
	 * Form submitted data.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private $request;

	/**
	 * Init class.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {

		if ( ! everest_backup_verify_nonce( $this->nonce_key ) ) {
			return;
		}

		$this->request = everest_backup_get_submitted_data();

	}

	/**
	 * Init parent class for migration key verification and key info extraction related work.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function handle_migration_key() {

		if ( empty( $this->request['verify_key'] ) ) {
			return;
		}

		if ( empty( $this->request['migration_key'] ) ) {
			return;
		}

		parent::__construct();

	}

	/**
	 * Download the package from the host site and start the clone.
	 *
	 * @return void
	 * @since 1.0.0
	 */
	public function handle_package_clone() {

		if ( empty( $this->request['clone_init'] ) ) {
			return;
		}

		if ( empty( $this->request['ebwp_cloner_download_url'] ) ) {
			return;
		}

		Logs::init( 'clone' );

		define( 'EVEREST_BACKUP_DOING_CLONE', true );
		define( 'EVEREST_BACKUP_DOING_ROLLBACK', true );

		$url = esc_url_raw( $this->request['ebwp_cloner_download_url'] );

		Logs::info( __( 'Downloading the file from the host site.', 'everest-backup' ) );

		$package = $this->download_package( $url );

		if ( ! $package ) {
			$message = __( 'Failed to download the file from the host site.', 'everest-backup' );
			Logs::error( $message );
			everest_backup_send_error( $message );
		}

		Logs::info( __( 'File downloaded successfully.', 'everest-backup' ) );

		$file = basename( $package );

		$_GET['file'] = $file;

		do_action( 'wp_ajax_everest_backup_import' ); // @phpcs:ignore

		Logs::info( __( 'Removing downloaded file.', 'everest-backup' ) );

		unlink( $package );

		if ( wp_safe_redirect( network_admin_url( '/admin.php?page=everest-backup-logs' ) ) ) {
			exit;
		}
	}

	/**
	 * Download the package from the host website.
	 *
	 * @param string $url Remote file url.
	 * @return string
	 * @since 1.0.0
	 */
	protected function download_package( $url ) {

		if ( ! $url ) {
			return;
		}

		$ssl = is_ssl() ? 'https://' : 'http://';

		$url = str_replace( array( 'https://', 'http://' ), $ssl, $url );

		$filename = basename( $url );

		$package = everest_backup_get_backup_full_path( $filename, false );

		if ( file_exists( $package ) ) {
			unlink( $package );
		}

		$bytes = file_put_contents( $package, fopen( $url, 'r' ) ); // @phpcs:ignore

		return is_int( $bytes ) && file_exists( $package ) ? $package : '';
	}

	/**
	 * Sets the generated migration key
	 *
	 * @return void
	 * @since 1.0.0
	 */
	protected function set_migration_key() {
		$this->migration_key = $this->request['migration_key'];
	}

	/**
	 * Check if user is trying to clone into same site.
	 *
	 * @param array $key_info Extracted information from migration key.
	 * @return boolean
	 * @since 1.0.2
	 */
	protected function is_same_site( $key_info ) {
		$siteurl = home_url( '/' );
		$url     = $key_info['url'];

		$explode = explode( $siteurl, $url );

		return count( $explode ) > 1;

	}

	/**
	 * Returns true if there's no error, returns error message otherwise.
	 *
	 * @return boolean|string
	 * @since 1.0.0
	 */
	public function is_clonable() {

		if ( ! $this->migration_key ) {
			return __( 'Cannot clone because migration key is not set.', 'everest-backup' );
		}

		$key_info = $this->get_key_info();

		if ( ! is_array( $key_info ) ) {
			return __( 'Unable to extract information from migration key.', 'everest-backup' );
		}

		if ( ! empty( $key_info['is_localhost'] ) ) {

			/**
			 * Error when user is trying to clone from localhost to live website.
			 */
			if ( ! everest_backup_is_localhost() ) {
				return __( 'Cannot clone to live website from localhost.', 'everest-backup' );
			}
		}

		if ( $this->is_same_site( $key_info ) ) {

			/**
			 * Error if user is trying to clone into same site.
			 */
			return __( 'Cannot clone to a same domain. Please enter the migration key in different domain for cloning.', 'everest-backup' );
		}

		$max_upload_size = everest_backup_max_upload_size();

		if ( $max_upload_size && ( $key_info['size'] >= $max_upload_size ) ) {
			return __( 'Cannot clone because package size is larger than allowed maximum upload size.', 'everest-backup' );
		}

		$disk_free_space = disk_free_space( EVEREST_BACKUP_BACKUP_DIR_PATH );
		$package_size    = $key_info['size'] * 1.5;

		if ( $package_size >= $disk_free_space ) {
			/* translators: %1$s is total filesize and %2$s is server available free disk space. */
			return sprintf( __( 'Required disk space: %1$s. Available disk space: %2$s.', 'everest-backup' ), esc_html( size_format( $package_size ) ), esc_html( size_format( $disk_free_space ) ) );
		}

		return true;

	}

}
