<?php
/**
 * Trait for restore module classes.
 *
 * @package everest-backup
 */

namespace Everest_Backup\Traits;

use Everest_Backup\Filesystem;

/**
 * Exit if accessed directly.
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Trait for restore module classes.
 *
 * @since 1.0.0
 */
trait Restore {

	/**
	 * Normalized path to the wp-content directory with trailing slash.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	private static $wp_content_dir;

	/**
	 * Returns path to storage directory with current export session uniqid.
	 *
	 * @var string
	 * @since 1.0.0
	 */
	private static $storage_dir;

	/**
	 * Filesystem class object.
	 *
	 * @var \Everest_Backup\Filesystem
	 * @since 1.0.0
	 */
	private static $filesystem;

	/**
	 * Extract class object.
	 *
	 * @var \Everest_Backup\Extract
	 * @since 1.0.0
	 */
	private static $extract;

	/**
	 * Array of files stored in root of zip.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private static $root;

	/**
	 * Array of files stored in nested folder of zip.
	 *
	 * @var array
	 * @since 1.0.0
	 */
	private static $nested;

	/**
	 * Init restore.
	 *
	 * @param \Everest_Backup\Extract $extract Extract class object.
	 * @return void
	 * @since 1.0.0
	 */
	public static function init( $extract ) {
		$filesystem = Filesystem::init();

		self::$wp_content_dir = trailingslashit( wp_normalize_path( WP_CONTENT_DIR ) );
		self::$storage_dir    = $extract->get_storage_dir();

		self::$extract    = $extract;
		self::$filesystem = $filesystem;

		$list = $extract->get_list();

		self::$root   = ! empty( $list['root'] ) ? $list['root'] : array();
		self::$nested = ! empty( $list['nested'] ) ? $list['nested'] : array();

		if ( method_exists( __CLASS__, 'before_restore' ) ) {
			self::before_restore();
		}

		self::restore();

		if ( method_exists( __CLASS__, 'after_restore' ) ) {
			self::after_restore();
		}

	}

	/**
	 * Returns config data for modules. It is only effective if it is called after config module.
	 *
	 * @return array
	 * @since 1.0.0
	 */
	protected static function get_config_data() {
		return self::$extract->get_temp_data( 'config_data' );
	}

	/**
	 * Check if current uploaded package is from single site or multisite.
	 *
	 * @return boolean
	 * @since 1.0.0
	 */
	protected static function is_package_multisite() {
		$config_data = self::get_config_data();

		return ! empty( $config_data['WordPress']['Multisite'] );
	}


	/**
	 * Returns file path of the matching filename from the array
	 *
	 * @param string $filename Name of the file to look into the $files.
	 * @param array  $files List of file.
	 * @param bool   $check [Optional] Check if file exists or not.
	 * @return string
	 * @since 1.0.0
	 */
	protected static function get_filepath_from_list( $filename, $files, $check = true ) {
		if ( ! $filename || ! $files ) {
			return;
		}

		if ( is_array( $files ) && ! empty( $files ) ) {
			foreach ( $files as $file ) {
				if ( false !== strpos( $file, $filename ) ) {

					if ( $check ) {
						return file_exists( $file ) ? $file : '';
					}

					return $file;
				}
			}
		}
	}

	/**
	 * Returns files from module/folder set inside Everest_Backup\Traits\Restore::$nested.
	 *
	 * @param string $module Module or the folder name from wp-content.
	 * @return array
	 * @since 1.0.0
	 */
	protected static function get_module_files( $module ) {
		$nested = self::$nested;

		if ( ! isset( $nested[ $module ] ) ) {
			return array();
		}

		return $nested[ $module ];
	}
}
